from __future__ import (absolute_import, division, print_function)

import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap as Basemap
prctile_rank = lambda x, p: np.searchsorted(np.percentile(x, np.atleast_1d(p)), x)

# cities colored by population rank.

m = Basemap()
shp_info = m.readshapefile('cities','cities')
x, y, pop = [], [], []
for item, (x_i, y_i) in zip(m.cities_info, m.cities):
    population = item['POPULATION']
    if population >= 0:
        pop.append(population)
        x.append(x_i)
        y.append(y_i)
popranks = prctile_rank(pop,np.linspace(0, 100, 101))
colors = []
for rank in popranks:
    colors.append(plt.cm.jet(float(rank)/100.))
m.drawcoastlines()
m.fillcontinents()
m.scatter(x,y,25,colors,marker='o',edgecolors='none',zorder=10)
plt.title('City Locations colored by Population Rank')
plt.show()



from __future__ import (absolute_import, division, print_function)

import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap as Basemap
from matplotlib.colors import rgb2hex, Normalize
from matplotlib.patches import Polygon
from matplotlib.colorbar import ColorbarBase

fig, ax = plt.subplots()

# Lambert Conformal map of lower 48 states.
m = Basemap(llcrnrlon=-119,llcrnrlat=20,urcrnrlon=-64,urcrnrlat=49,
            projection='lcc',lat_1=33,lat_2=45,lon_0=-95)

# Mercator projection, for Alaska and Hawaii
m_ = Basemap(llcrnrlon=-190,llcrnrlat=20,urcrnrlon=-143,urcrnrlat=46,
            projection='merc',lat_ts=20)  # do not change these numbers

#%% ---------   draw state boundaries  ----------------------------------------
## data from U.S Census Bureau
## http://www.census.gov/geo/www/cob/st2000.html
shp_info = m.readshapefile('st99_d00','states',drawbounds=True,
                           linewidth=0.45,color='gray')
shp_info_ = m_.readshapefile('st99_d00','states',drawbounds=False)

## population density by state from
## http://en.wikipedia.org/wiki/List_of_U.S._states_by_population_density
popdensity = {
'New Jersey':  438.00,
'Rhode Island':   387.35,
'Massachusetts':   312.68,
'Connecticut':	  271.40,
'Maryland':   209.23,
'New York':    155.18,
'Delaware':    154.87,
'Florida':     114.43,
'Ohio':	 107.05,
'Pennsylvania':	 105.80,
'Illinois':    86.27,
'California':  83.85,
'Hawaii':  72.83,
'Virginia':    69.03,
'Michigan':    67.55,
'Indiana':    65.46,
'North Carolina':  63.80,
'Georgia':     54.59,
'Tennessee':   53.29,
'New Hampshire':   53.20,
'South Carolina':  51.45,
'Louisiana':   39.61,
'Kentucky':   39.28,
'Wisconsin':  38.13,
'Washington':  34.20,
'Alabama':     33.84,
'Missouri':    31.36,
'Texas':   30.75,
'West Virginia':   29.00,
'Vermont':     25.41,
'Minnesota':  23.86,
'Mississippi':	 23.42,
'Iowa':	 20.22,
'Arkansas':    19.82,
'Oklahoma':    19.40,
'Arizona':     17.43,
'Colorado':    16.01,
'Maine':  15.95,
'Oregon':  13.76,
'Kansas':  12.69,
'Utah':	 10.50,
'Nebraska':    8.60,
'Nevada':  7.03,
'Idaho':   6.04,
'New Mexico':  5.79,
'South Dakota':	 3.84,
'North Dakota':	 3.59,
'Montana':     2.39,
'Wyoming':      1.96,
'Alaska':     0.42}

#%% -------- choose a color for each state based on population density. -------
colors={}
statenames=[]
cmap = plt.cm.hot_r # use 'reversed hot' colormap
vmin = 0; vmax = 450 # set range.
norm = Normalize(vmin=vmin, vmax=vmax)
for shapedict in m.states_info:
    statename = shapedict['NAME']
    # skip DC and Puerto Rico.
    if statename not in ['District of Columbia','Puerto Rico']:
        pop = popdensity[statename]
        # calling colormap with value between 0 and 1 returns
        # rgba value.  Invert color range (hot colors are high
        # population), take sqrt root to spread out colors more.
        colors[statename] = cmap(np.sqrt((pop-vmin)/(vmax-vmin)))[:3]
    statenames.append(statename)

#%% ---------  cycle through state names, color each one.  --------------------
for nshape,seg in enumerate(m.states):
    # skip DC and Puerto Rico.
    if statenames[nshape] not in ['Puerto Rico', 'District of Columbia']:
        color = rgb2hex(colors[statenames[nshape]])
        poly = Polygon(seg,facecolor=color,edgecolor=color)
        ax.add_patch(poly)

AREA_1 = 0.005  # exclude small Hawaiian islands that are smaller than AREA_1
AREA_2 = AREA_1 * 30.0  # exclude Alaskan islands that are smaller than AREA_2
AK_SCALE = 0.19  # scale down Alaska to show as a map inset
HI_OFFSET_X = -1900000  # X coordinate offset amount to move Hawaii "beneath" Texas
HI_OFFSET_Y = 250000    # similar to above: Y offset for Hawaii
AK_OFFSET_X = -250000   # X offset for Alaska (These four values are obtained
AK_OFFSET_Y = -750000   # via manual trial and error, thus changing them is not recommended.)

for nshape, shapedict in enumerate(m_.states_info):  # plot Alaska and Hawaii as map insets
    if shapedict['NAME'] in ['Alaska', 'Hawaii']:
        seg = m_.states[int(shapedict['SHAPENUM'] - 1)]
        if shapedict['NAME'] == 'Hawaii' and float(shapedict['AREA']) > AREA_1:
            seg = [(x + HI_OFFSET_X, y + HI_OFFSET_Y) for x, y in seg]
            color = rgb2hex(colors[statenames[nshape]])
        elif shapedict['NAME'] == 'Alaska' and float(shapedict['AREA']) > AREA_2:
            seg = [(x*AK_SCALE + AK_OFFSET_X, y*AK_SCALE + AK_OFFSET_Y)\
                   for x, y in seg]
            color = rgb2hex(colors[statenames[nshape]])
        poly = Polygon(seg, facecolor=color, edgecolor='gray', linewidth=.45)
        ax.add_patch(poly)

ax.set_title('United states population density by state')

#%% ---------  Plot bounding boxes for Alaska and Hawaii insets  --------------
light_gray = [0.8]*3  # define light gray color RGB
x1,y1 = m_([-190,-183,-180,-180,-175,-171,-171],[29,29,26,26,26,22,20])
x2,y2 = m_([-180,-180,-177],[26,23,20])  # these numbers are fine-tuned manually
m_.plot(x1,y1,color=light_gray,linewidth=0.8)  # do not change them drastically
m_.plot(x2,y2,color=light_gray,linewidth=0.8)

#%% ---------   Show color bar  ---------------------------------------
ax_c = fig.add_axes([0.9, 0.1, 0.03, 0.8])
cb = ColorbarBase(ax_c,cmap=cmap,norm=norm,orientation='vertical',
                  label=r'[population per $\mathregular{km^2}$]')

plt.show()




from __future__ import (absolute_import, division, print_function)

from mpl_toolkits.basemap import Basemap, shiftgrid, maskoceans, interp
import numpy as np 
import matplotlib.pyplot as plt

# example showing how to mask out 'wet' areas on a contour or pcolor plot.

topodatin = np.loadtxt('etopo20data.gz')
lonsin = np.loadtxt('etopo20lons.gz')
latsin = np.loadtxt('etopo20lats.gz')

# shift data so lons go from -180 to 180 instead of 20 to 380.
topoin,lons1 = shiftgrid(180.,topodatin,lonsin,start=False)
lats1 = latsin

fig=plt.figure()
# setup basemap
m=Basemap(resolution='l',projection='lcc',lon_0=-100,lat_0=40,width=8.e6,height=6.e6)
lons, lats = np.meshgrid(lons1,lats1)
x, y = m(lons, lats)
# interpolate land/sea mask to topo grid, mask ocean values.
# output may look 'blocky' near coastlines, since data is at much
# lower resolution than land/sea mask.
topo = maskoceans(lons, lats, topoin)
# make contour plot (ocean values will be masked)
CS=m.contourf(x,y,topo,np.arange(-300,3001,50),cmap=plt.cm.jet,extend='both')
#im=m.pcolormesh(x,y,topo,cmap=plt.cm.jet,vmin=-300,vmax=3000)
# draw coastlines.
m.drawcoastlines()
plt.title('ETOPO data with marine areas masked (original grid)')

fig=plt.figure()
# interpolate topo data to higher resolution grid (to better match
# the land/sea mask). Output looks less 'blocky' near coastlines.
nlats = 3*topoin.shape[0]
nlons = 3*topoin.shape[1]
lons = np.linspace(-180,180,nlons)
lats = np.linspace(-90,90,nlats)
lons, lats = np.meshgrid(lons, lats)
x, y = m(lons, lats)
topo = interp(topoin,lons1,lats1,lons,lats,order=1)
# interpolate land/sea mask to topo grid, mask ocean values.
topo = maskoceans(lons, lats, topo)
# make contour plot (ocean values will be masked)
CS=m.contourf(x,y,topo,np.arange(-300,3001,50),cmap=plt.cm.jet,extend='both')
#im=m.pcolormesh(x,y,topo,cmap=plt.cm.jet,vmin=-300,vmax=3000)
# draw coastlines.
m.drawcoastlines()
plt.title('ETOPO data with marine areas masked (data on finer grid)')
plt.show()



from mpl_toolkits.basemap import Basemap
import numpy as np
import matplotlib.pyplot as plt
lon_0 = -105; lat_0 = 40
m = Basemap(projection='aeqd',lat_0=lat_0,lon_0=lon_0)
# fill background.
m.drawmapboundary(fill_color='aqua')
# draw coasts and fill continents.
m.drawcoastlines(linewidth=0.5)
m.fillcontinents(color='coral',lake_color='aqua')
# 20 degree graticule.
m.drawparallels(np.arange(-80,81,20))
m.drawmeridians(np.arange(-180,180,20))
# draw a black dot at the center.
xpt, ypt = m(lon_0, lat_0)
m.plot([xpt],[ypt],'ko') 
# draw the title.
plt.title('Whole World Azimuthal Equidistant Projection')
plt.show()



from __future__ import (absolute_import, division, print_function)

from netCDF4 import Dataset
from mpl_toolkits.basemap import Basemap
import numpy as np
import matplotlib.pyplot as plt

nc = Dataset('wm201_Arctic_JJA_1990-2008_moyenneDesMoyennes.nc')
lats = nc.variables['lat'][:]
lons = nc.variables['lon'][:]
rlats = nc.variables['rlat'][:]
rlons = nc.variables['rlon'][:]
rlons, rlats = np.meshgrid(rlons, rlats)
data = nc.variables['air'][0,0,:,:].squeeze()
data = np.ma.masked_values(data,-999.)
rotpole = nc.variables['rotated_pole']

m = Basemap(projection='npstere',lon_0=10,boundinglat=30,resolution='c')
x,y = m(lons,lats)
m.drawcoastlines()
m.contourf(x,y,data,20)
m.drawmeridians(np.arange(-180,180,20))
m.drawparallels(np.arange(20,80,20))
m.colorbar()
plt.title('rotated pole data in polar stere map')

plt.figure()

# o_lon_p, o_lat_p: true lat/lon of pole in rotated coordinate system

# mapping to CF metadata convention:
# grid_north_pole_longitude = normalize180(180 + lon_0), where normalize180
#  is a function that maps to interval [-180,180).
# grid_north_pole_latitude = o_lat_p
# north_pole_grid_longitude = o_lon_p (optional, assumed zero if not present)

def normalize180(lon):
    """Normalize lon to range [180, 180)"""
    lower = -180.; upper = 180.
    if lon > upper or lon == lower:
        lon = lower + abs(lon + upper) % (abs(lower) + abs(upper))
    if lon < lower or lon == upper:
        lon = upper - abs(lon - lower) % (abs(lower) + abs(upper))
    return lower if lon == upper else lon

lon_0 = normalize180(rotpole.grid_north_pole_longitude-180.)
o_lon_p = rotpole.north_pole_grid_longitude
o_lat_p = rotpole.grid_north_pole_latitude
print( rotpole )
print( 'lon_0,o_lon_p,o_lat_p=',lon_0,o_lon_p,o_lat_p)

m= Basemap(projection='rotpole',lon_0=lon_0,o_lon_p=o_lon_p,o_lat_p=o_lat_p,\
           llcrnrlat = lats[0,0], urcrnrlat = lats[-1,-1],\
           llcrnrlon = lons[0,0], urcrnrlon = lons[-1,-1],resolution='c')
x,y = m(lons,lats)
m.drawcoastlines()
m.contourf(x,y,data,20)
m.drawmeridians(np.arange(-180,180,20))
m.drawparallels(np.arange(20,80,20))
m.colorbar()
plt.title('rotated pole data in native map using real sphere corner lat/lons' )

plt.figure()
m= Basemap(projection='rotpole',lon_0=lon_0,o_lon_p=o_lon_p,o_lat_p=o_lat_p,\
           llcrnry = rlats[0,0], urcrnry = rlats[-1,-1],\
           llcrnrx = rlons[0,0], urcrnrx = rlons[-1,-1],resolution='c')
x,y = m(lons,lats)
m.drawcoastlines()
m.contourf(x,y,data,20)
m.drawmeridians(np.arange(-180,180,20))
m.drawparallels(np.arange(20,80,20))
m.colorbar()
plt.title('rotated pole data in native map using rotated sphere corner lat/lons' )

plt.show()



import sys
import numpy as np

lsd = 3

def quantize(data,least_significant_digit):
    """
    Quantize data to improve compression. data is quantized using
    around(scale*data)/scale, where scale is 2**bits, and bits is determined
    from the least_significant_digit. For example, if
    least_significant_digit=1, bits will be 4.

    This function is pure python.
    """
    precision = pow(10.,-least_significant_digit)
    exp = np.log10(precision)
    if exp < 0:
        exp = int(np.floor(exp))
    else:
        exp = int(np.ceil(exp))
    bits = np.ceil(np.log2(pow(10.,-exp)))
    scale = pow(2.,bits)
    return np.around(scale*data)/scale

def interpolate_long_segments(coords, resolution):
    lookup_thresh = {'c': 0.5, 'l':0.3, 'i':0.2, 'h':0.1, 'f':0.05}
    thresh = lookup_thresh[resolution]
    spacing = thresh / 5.0

    lons, lats = coords.T
    dist = np.hypot(np.diff(lons), np.diff(lats))

    if np.all(dist <= thresh):
        return coords

    out_lon, out_lat = [], []
    for i in np.arange(len(dist)):
        if dist[i] <= thresh:
            out_lon.append(lons[i])
            out_lat.append(lats[i])
        else:
            x = [0, dist[i]]
            new_x = np.arange(0, dist[i], spacing)
            out_lon.extend(np.interp(new_x, x, lons[i:i+2]))
            out_lat.extend(np.interp(new_x, x, lats[i:i+2]))

    out_lon.append(lons[-1])
    out_lat.append(lats[-1])
    return np.column_stack([out_lon, out_lat]).astype(coords.dtype)

def get_coast_polygons(coastfile):
    polymeta = []; polybounds = []
    lats = []; lons = []
    for line in open(coastfile):
        if line.startswith('#'):
            continue
        linesplit = line.strip().split()
        if line.startswith('>'):
            area, west, east, south, north = map(float, linesplit[5:10])
            poly_id = linesplit[-1]
            level = linesplit[3]
            polymeta.append([level,area,south,north,poly_id])
            if lons:
                #lons.append(lons[0]); lats.append(lats[0])
                b = np.empty((len(lons),2),np.float32)
                b[:,0] = lons; b[:,1] = lats
                if lsd is not None:
                    b = quantize(b,lsd)
                polybounds.append(b)
            lats = []; lons = []
            continue
        lon = float(linesplit[0])
        lat = float(linesplit[1])
        lons.append(lon); lats.append(lat)
    #lons.append(lons[0]); lats.append(lats[0])
    b = np.empty((len(lons),2),np.float32)
    b[:,0] = lons; b[:,1] = lats
    if lsd is not None:
        b = quantize(b,lsd)
    polybounds.append(b)
    polymeta2 = []
    for meta,bounds in zip(polymeta,polybounds):
        npts = bounds.shape[0]
        polymeta2.append(meta[:-1] + [npts] + [meta[-1]])
    return polybounds, polymeta2

def get_boundary_lines(bdatfile, resolution):
    lons = []; lats = []; polybounds = []
    for line in open(bdatfile):
        if line.startswith('#'): continue
        linesplit = line.split()
        if line.startswith('>'):
           if lons:
               b = np.empty((len(lons),2),np.float32)
               b[:,0] = lons; b[:,1] = lats
               b = interpolate_long_segments(b, resolution)
               if lsd is not None:
                   b = quantize(b,lsd)
               polybounds.append(b)
           lons = []; lats = []
           continue
        lon, lat = [float(val) for val in linesplit]
        lats.append(lat); lons.append(lon)
    b = np.empty((len(lons),2),np.float32)
    b[:,0] = lons; b[:,1] = lats
    b = interpolate_long_segments(b, resolution)
    if lsd is not None:
        b = quantize(b,lsd)
    polybounds.append(b)
    polymeta = []
    polybounds2 = []
    for bounds in polybounds:
        npts = bounds.shape[0]
        if npts == 2 and\
           bounds[0,0] == bounds[1,0] and\
           bounds[0,1] == bounds[1,1]: continue
        polybounds2.append(bounds)
        south = bounds[:,1].min()
        north = bounds[:,1].max()
        polymeta.append((south,north,npts))
    return polybounds2, polymeta

# read in coastline data (only those polygons whose area > area_thresh).
for resolution in ['c','l','i','h','f']:
    coastlons = []; coastlats = []; coastsegind = []; coastsegtype = []
    coastfile = 'gshhs_'+resolution+'.txt'
    countryfile = 'countries_'+resolution+'.txt'
    statefile = 'states_'+resolution+'.txt'
    riverfile = 'rivers_'+resolution+'.txt'

    poly, polymeta = get_coast_polygons(coastfile)
    f = open('../lib/mpl_toolkits/basemap/data/gshhs_'+resolution+'.dat','wb')
    f2 = open('../lib/mpl_toolkits/basemap/data/gshhsmeta_'+resolution+'.dat','w')
    offset = 0
    for p,pm in zip(poly,polymeta):
        bstring = p.tostring()
        f.write(bstring)
        typ = pm[0]; area = pm[1]; south = pm[2]; north = pm[3]; npts = pm[4]
        poly_id = pm[5]
        f2.write('%s %s %s %9.5f %9.5f %s %s %s\n' % (typ, area, npts, south, north, offset, len(bstring), poly_id))
        offset = offset + len(bstring)
    f.close()
    f2.close()

    poly, polymeta = get_boundary_lines(countryfile, resolution)
    f = open('../lib/mpl_toolkits/basemap/data/countries_'+resolution+'.dat','wb')
    f2 = open('../lib/mpl_toolkits/basemap/data/countriesmeta_'+resolution+'.dat','w')
    offset = 0
    for p,pm in zip(poly,polymeta):
        bstring = p.tostring()
        f.write(bstring)
        south,north,npts = pm[:]
        f2.write('%s %s %s %9.5f %9.5f %s %s\n' % (-1,-1,npts, south, north, offset, len(bstring)))
        offset = offset + len(bstring)
    f.close()
    f2.close()

    poly, polymeta = get_boundary_lines(statefile, resolution)
    f = open('../lib/mpl_toolkits/basemap/data/states_'+resolution+'.dat','wb')
    f2 = open('../lib/mpl_toolkits/basemap/data/statesmeta_'+resolution+'.dat','w')
    offset = 0
    for p,pm in zip(poly,polymeta):
        bstring = p.tostring()
        f.write(bstring)
        south,north,npts = pm[:]
        f2.write('%s %s %s %9.5f %9.5f %s %s\n' % (-1,-1,npts, south, north, offset, len(bstring)))
        offset = offset + len(bstring)
    f.close()
    f2.close()

    poly, polymeta = get_boundary_lines(riverfile, resolution)
    f = open('../lib/mpl_toolkits/basemap/data/rivers_'+resolution+'.dat','wb')
    f2 = open('../lib/mpl_toolkits/basemap/data/riversmeta_'+resolution+'.dat','w')
    offset = 0
    for p,pm in zip(poly,polymeta):
        bstring = p.tostring()
        f.write(bstring)
        south,north,npts = pm[:]
        f2.write('%s %s %s %9.5f %9.5f %s %s\n' % (-1,-1,npts, south, north, offset, len(bstring)))
        offset = offset + len(bstring)
    f.close()
    f2.close()



"""
draw Atlantic Hurricane Tracks for storms that reached Cat 4 or 5.
part of the track for which storm is cat 4 or 5 is shown red.
ESRI shapefile data from http://nationalatlas.gov/mld/huralll.html
"""
import os
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
# Lambert Conformal Conic map.
m = Basemap(llcrnrlon=-100.,llcrnrlat=0.,urcrnrlon=-20.,urcrnrlat=57.,
            projection='lcc',lat_1=20.,lat_2=40.,lon_0=-60.,
            resolution ='l',area_thresh=1000.)
# read shapefile.
shp_path = os.path.join(*3 * [".."] + ["examples", "huralll020"])
shp_info = m.readshapefile(shp_path,'hurrtracks',drawbounds=False)
# find names of storms that reached Cat 4.
names = []
for shapedict in m.hurrtracks_info:
    cat = shapedict['CATEGORY']
    name = shapedict['NAME']
    if cat in ['H4','H5'] and name not in names:
        # only use named storms.
        if name != 'NOT NAMED':  names.append(name)
# plot tracks of those storms.
for shapedict,shape in zip(m.hurrtracks_info,m.hurrtracks):
    name = shapedict['NAME']
    cat = shapedict['CATEGORY']
    if name in names:
        xx,yy = zip(*shape)
        # show part of track where storm > Cat 4 as thick red.
        if cat in ['H4','H5']: 
            m.plot(xx,yy,linewidth=1.5,color='r')
        elif cat in ['H1','H2','H3']:
            m.plot(xx,yy,color='k')
# draw coastlines, meridians and parallels.
m.drawcoastlines()
m.drawcountries()
m.drawmapboundary(fill_color='#99ffff')
m.fillcontinents(color='#cc9966',lake_color='#99ffff')
m.drawparallels(np.arange(10,70,20),labels=[1,1,0,0])
m.drawmeridians(np.arange(-100,0,20),labels=[0,0,0,1])
plt.title('Atlantic Hurricane Tracks (Storms Reaching Category 4, 1851-2004)')
plt.show()



from __future__ import (absolute_import, division, print_function)

from __future__ import unicode_literals
"""
AllSkyMap is a subclass of Basemap, specialized for handling common plotting
tasks for celestial data.

It is essentially equivalent to using Basemap with full-sphere projections
(e.g., 'hammer' or 'moll') and the `celestial` keyword set to `True`, but
it adds a few new methods:

* label_meridians for, well, labeling meridians with their longitude values;

* geodesic, a replacement for Basemap.drawgreatcircle, that can correctly
  handle geodesics that cross the limb of the map, and providing the user
  easy control over clipping (which affects thick lines at or near the limb);
  
* tissot, which overrides Basemap.tissot, correctly handling geodesics that
  cross the limb of the map.

Created Jan 2011 by Tom Loredo, based on Jeff Whitaker's code in Basemap's
__init__.py module.
"""

from numpy import *
import matplotlib.pyplot as pl
from matplotlib.pyplot import *
from mpl_toolkits.basemap import Basemap
import pyproj
from pyproj import Geod

__all__ = ['AllSkyMap']

def angle_symbol(angle, round_to=1.0):
    """
    Return a string representing an angle, rounded and with a degree symbol.
    
    This is adapted from code in mpl's projections.geo module.
    """
    value = np.round(angle / round_to) * round_to
    if pl.rcParams['text.usetex'] and not pl.rcParams['text.latex.unicode']:
        return r'$%0.0f^\circ$' % value
    else:
        return '%0.0f\N{DEGREE SIGN}' % value


class AllSkyMap(Basemap):
    """
    AllSkyMap is a subclass of Basemap, specialized for handling common plotting
    tasks for celestial data.
    
    It is essentially equivalent to using Basemap with full-sphere projections
    (e.g., 'hammer' or 'moll') and the `celestial` keyword set to `True`, but
    it adds a few new methods:
    
    * label_meridians for, well, labeling meridians with their longitude values;
    
    * geodesic, a replacement for Basemap.drawgreatcircle, that can correctly
      handle geodesics that cross the limb of the map, and providing the user
      easy control over clipping (which affects thick lines at or near the
      limb);
      
    * tissot, which overrides Basemap.tissot, correctly handling geodesics that
      cross the limb of the map.
    """

    # Longitudes corresponding to east and west edges, reflecting the
    # convention that 180 deg is the eastern edge, according to basemap's 
    # underlying projections:
    east_lon = 180.
    west_lon = 180.+1.e-10

    def __init__(self, 
                       projection='hammer',
                       lat_0=0., lon_0=0.,
                       suppress_ticks=True,
                       boundinglat=None,
                       fix_aspect=True,
                       anchor=str('C'),
                       ax=None):

        if projection != 'hammer' and projection !='moll':
            raise ValueError('Only hammer and moll projections supported!')

        # Use Basemap's init, enforcing the values of many parameters that
        # aren't used or whose Basemap defaults would not be altered for all-sky
        # celestial maps.
        Basemap.__init__(self, llcrnrlon=None, llcrnrlat=None,
                       urcrnrlon=None, urcrnrlat=None,
                       llcrnrx=None, llcrnry=None,
                       urcrnrx=None, urcrnry=None,
                       width=None, height=None,
                       projection=projection, resolution=None,
                       area_thresh=None, rsphere=1.,
                       lat_ts=None,
                       lat_1=None, lat_2=None,
                       lat_0=lat_0, lon_0=lon_0,
                       suppress_ticks=suppress_ticks,
                       satellite_height=1.,
                       boundinglat=None,
                       fix_aspect=True,
                       anchor=anchor,
                       celestial=True,
                       ax=ax)

        # Keep a local ref to lon_0 for hemisphere checking.
        self._lon_0 = self.projparams['lon_0']
        self._limb = None

    def drawmapboundary(self,color='k',linewidth=1.0,fill_color=None,\
                        zorder=None,ax=None):
        """
        draw boundary around map projection region, optionally
        filling interior of region.

        .. tabularcolumns:: |l|L|

        ==============   ====================================================
        Keyword          Description
        ==============   ====================================================
        linewidth        line width for boundary (default 1.)
        color            color of boundary line (default black)
        fill_color       fill the map region background with this
                         color (default is no fill or fill with axis
                         background color).
        zorder           sets the zorder for filling map background
                         (default 0).
        ax               axes instance to use
                         (default None, use default axes instance).
        ==============   ====================================================

        returns matplotlib.collections.PatchCollection representing map boundary.
        """
        # Just call the base class version, but keep a copy of the limb
        # polygon for clipping.
        self._limb = Basemap.drawmapboundary(self, color=color,
            linewidth=linewidth, fill_color=fill_color, zorder=zorder, ax=ax)
        return self._limb

    def label_meridians(self, lons, fontsize=10, valign='bottom', vnudge=0,
                        halign='center', hnudge=0):
        """
        Label meridians with their longitude values in degrees.
        
        This labels meridians with negative longitude l with the value 360-l;
        for maps in celestial orientation, this means meridians to the right
        of the central meridian are labeled from 360 to 180 (left to right).
        
        `vnudge` and `hnudge` specify amounts in degress to nudge the labels
        from their default placements, vertically and horizontally.  This
        values obey the map orientation, so to nudge to the right, use a
        negative `hnudge` value.
        """
        # Run through (lon, lat) pairs, with lat=0 in each pair.
        lats = len(lons)*[0.]
        for lon,lat in zip(lons, lats):
            x, y = self(lon+hnudge, lat+vnudge)
            if lon < 0:
                lon_lbl = 360 + lon
            else:
                lon_lbl = lon
            pl.text(x, y, angle_symbol(lon_lbl), fontsize=fontsize,
                    verticalalignment=valign,
                    horizontalalignment=halign)

    def east_hem(self, lon):
        """
        Return True if lon is in the eastern hemisphere of the map wrt lon_0.
        """
        if (lon-self._lon_0) % 360. <= self.east_lon:
            return True
        else:
            return False

    def geodesic(self, lon1, lat1, lon2, lat2, del_s=.01, clip=True, **kwargs):
        """
        Plot a geodesic curve from (lon1, lat1) to (lon2, lat2), with
        points separated by arc length del_s.  Return a list of Line2D
        instances for the curves comprising the geodesic.  If the geodesic does
        not cross the map limb, there will be only a single curve; if it
        crosses the limb, there will be two curves.
        """
        
        # TODO:  Perhaps return a single Line2D instance when there is only a
        # single segment, and a list of segments only when there are two segs?

        # TODO:  Check the units of del_s.
        
        # This is based on Basemap.drawgreatcircle (which draws an *arc* of a
        # great circle), but addresses a limitation of that method, supporting
        # geodesics that cross the map boundary by breaking them into two
        # segments, one in the eastern hemisphere and the other in the western.
        gc = pyproj.Geod(a=self.rmajor,b=self.rminor)
        az12,az21,dist = gc.inv(lon1,lat1,lon2,lat2)
        npoints = int((dist+0.5**del_s)/del_s)
        # Calculate lon & lat for points on the arc.
        lonlats = gc.npts(lon1,lat1,lon2,lat2,npoints)
        lons = [lon1]; lats = [lat1]
        for lon, lat in lonlats:
            lons.append(lon)
            lats.append(lat)
        lons.append(lon2); lats.append(lat2)
        # Break the arc into segments as needed, when there is a longitudinal
        # hemisphere crossing.
        segs = []
        seg_lons, seg_lats = [lon1], [lat1]
        cur_hem = self.east_hem(lon1)
        for lon, lat in zip(lons[1:], lats[1:]):
            if self.east_hem(lon) == cur_hem:
                seg_lons.append(lon)
                seg_lats.append(lat)
            else:
                # We should interpolate a new pt at the boundary, but in
                # the mean time just rely on the step size being small.
                segs.append( (seg_lons, seg_lats) )
                seg_lons, seg_lats = [lon], [lat]
                cur_hem = not cur_hem
        segs.append( (seg_lons, seg_lats) )
        # Plot each segment; return a list of the mpl lines.
        lines = []
        for lons, lats in segs:
            x, y = self(lons, lats)
            if clip and self._limb:
                line = plot(x, y, clip_path=self._limb, **kwargs)[0]
            else:
                line = plot(x, y, **kwargs)[0]
            lines.append(line)
        # If there are multiple segments and no color args, reconcile the
        # colors, which mpl will have autoset to different values.
        # *** Does this screw up mpl's color set sequence for later lines?
        if 'c' not in kwargs or 'color' in kwargs:
            if len(lines) > 1:
                c1 = lines[0].get_color()
                for line in lines[1:]:
                    line.set_color(c1)
        return lines

    def tissot(self,lon_0,lat_0,radius_deg,npts,ax=None,**kwargs):
        """
        Draw a polygon centered at ``lon_0,lat_0``.  The polygon
        approximates a circle on the surface of the earth with radius
        ``radius_deg`` degrees latitude along longitude ``lon_0``,
        made up of ``npts`` vertices.
        
        The polygon represents a Tissot's indicatrix
        (http://en.wikipedia.org/wiki/Tissot's_Indicatrix),
        which when drawn on a map shows the distortion inherent in the map
        projection.  Tissots can be used to display azimuthally symmetric
        directional uncertainties ("error circles").

        Extra keyword ``ax`` can be used to override the default axis instance.

        Other \**kwargs passed on to matplotlib.patches.Polygon.

        returns a list of matplotlib.patches.Polygon objects, with two polygons
        when the tissot crosses the limb, and just one polygon otherwise.
        """
        
        # TODO:  Just return the polygon (not a list) when there is only one
        # polygon?  Or stick with the list for consistency?
        
        # This is based on Basemap.tissot, but addresses a limitation of that
        # method by handling tissots that cross the limb of the map by finding
        # separate polygons in the eastern and western hemispheres comprising
        # the tissot.
        ax = kwargs.pop('ax', None) or self._check_ax()
        g = pyproj.Geod(a=self.rmajor,b=self.rminor)
        az12,az21,dist = g.inv(lon_0,lat_0,lon_0,lat_0+radius_deg)
        start_hem = self.east_hem(lon_0)
        segs1 = [self(lon_0,lat_0+radius_deg)]
        over, segs2 = [], []
        delaz = 360./npts
        az = az12
        last_lon = lon_0
        # Note adjacent and opposite edge longitudes, in case the tissot
        # runs over the edge.
        if start_hem:  # eastern case
            adj_lon = self.east_lon
            opp_lon = self.west_lon
        else:
            adj_lon = self.west_lon
            opp_lon = self.east_lon
        for n in range(npts):
            az = az+delaz
            # skip segments along equator (Geod can't handle equatorial arcs)
            if np.allclose(0.,lat_0) and (np.allclose(90.,az) or np.allclose(270.,az)):
                continue
            else:
                lon, lat, az21 = g.fwd(lon_0, lat_0, az, dist)
            # If in the starting hemisphere, add to 1st polygon seg list.
            if self.east_hem(lon) == start_hem:
                x, y = self(lon, lat)
                # Add segment if it is in the map projection region.
                if x < 1.e20 and y < 1.e20:
                    segs1.append( (x, y) )
                    last_lon = lon
            # Otherwise, we cross hemispheres.
            else:
                # Trace the edge of each hemisphere.
                x, y = self(adj_lon, lat)
                if x < 1.e20 and y < 1.e20:
                    segs1.append( (x, y) )
                    # We presume if adj projection is okay, opposite is.
                    segs2.append( self(opp_lon, lat) )
                # Also store the overlap in the opposite hemisphere.
                x, y = self(lon, lat)
                if x < 1.e20 and y < 1.e20:
                    over.append( (x, y) )
                    last_lon = lon
        poly1 = Polygon(segs1, **kwargs)
        ax.add_patch(poly1)
        if segs2:
            over.reverse()
            segs2.extend(over)
            poly2 = Polygon(segs2, **kwargs)
            ax.add_patch(poly2)
            return [poly1, poly2]
        else:
            return [poly1]


if __name__ == '__main__':

    # Note that Hammer & Mollweide projections enforce a 2:1 aspect ratio.
    # Use figure size good for a 2:1 plot.
    fig = figure(figsize=(12,6))
    
    # Set up the projection and draw a grid.
    map = AllSkyMap(projection='hammer')
    # Save the bounding limb to use as a clip path later.
    limb = map.drawmapboundary(fill_color='white')
    map.drawparallels(np.arange(-75,76,15), linewidth=0.5, dashes=[1,2],
        labels=[1,0,0,0], fontsize=9)
    map.drawmeridians(np.arange(-150,151,30), linewidth=0.5, dashes=[1,2])
    
    # Label a subset of meridians.
    lons = np.arange(-150,151,30)
    map.label_meridians(lons, fontsize=9, vnudge=1,
                    halign='left', hnudge=-1)  # hnudge<0 shifts to right
    
    # x, y limits are [0, 4*rt2], [0, 2*rt2].
    rt2 = sqrt(2)

    # Draw a slanted green line crossing the map limb.
    line = plot([rt2,0], [rt2,2*rt2], 'g-')

    # Draw a slanted magenta line crossing the map limb but clipped.
    line = plot([rt2+.1,0+.1], [rt2,2*rt2], 'm-', clip_path=limb)
    
    # Draw some geodesics.
    # First a transparent thick blue geodesic crossing the limb but not clipped,
    # overlayed by a thinner red geodesic that is clipped (by default), to
    # illustrate the effect of clipping.
    lines = map.geodesic(120, 30, 240, 60, clip=False, c='b', lw=7, alpha=.5)
    lines = map.geodesic(240, 60, 120, 30, c='r', lw=3, alpha=.5)

    # Next two large limb-crossing geodesics with the same path, but rendered
    # in opposite directions, one transparent blue, the other transparent
    # yellow.  They should be right on top of each other, giving a greenish
    # brown hue.
    lines = map.geodesic(240, -60, 120, 30, c='b', lw=2, alpha=.5)
    lines = map.geodesic(120, 30, 240, -60, c='y', lw=2, alpha=.5)

    # What happens if a geodesic is given coordinates spanning more than
    # a single rotation?  Not sure what to expect, but it shoots off the
    # map (clipped here).  Perhaps we should ensure lons are in [0, 360].
    #lines = map.geodesic(120, 20, 240+360, 50, del_s=.2, c='g')
    
    # Two tissots fully within the limb.
    poly = map.tissot(60, -15, 10, 100)
    poly = map.tissot(280, 60, 10, 100)
    #poly = map.tissot(90, -85, 10, 100)
    
    # Limb-spanning tissots in each quadrant.
    # lower left:
    poly = map.tissot(170, -60, 15, 100)
    # upper left:
    poly = map.tissot(175, 70, 15, 100)
    # upper right (note negative longitude):
    poly = map.tissot(-175, 30, 15, 100, color='r', alpha=.6)
    # lower right:
    poly = map.tissot(185, -40, 10, 100)

    # Plot the tissot centers as "+" symbols.  Note the top left symbol
    # would cross the limb without the clip_path argument; this might be
    # desired to enhance visibility.
    lons = [170, 175, -175, 185]
    lats = [-60, 70, 30, -40]
    x, y = map(lons, lats)
    map.scatter(x, y, s=40, marker='+', linewidths=1, edgecolors='g',
        facecolors='none', clip_path=limb, zorder=10)  # hi zorder -> top
    
    title('AllSkyMap demo:  Clipped lines, markers, geodesics, tissots')
    show()



"""Simple functions to calculate solar position and day-night terminator."""
# pylint: disable=invalid-name
from __future__ import division

import numpy as np


def JulianDayFromDate(date, calendar="standard"):
    """Return Julian day from a :class:`datetime.datetime` object.

    Algorithm:

        Meeus, Jean (1998) Astronomical Algorithms (2nd Edition).
        Willmann-Bell, Virginia. p. 63

    Paramaters
    ---------

    date : datetime.datetime
        a :class:`datetime.datetime` object

    calendar : {'standard', 'gregorian', 'proleptic_gregorian',
                'julian'}, optional
        if 'standard' or 'gregorian', Julian day follows the Julian
        calendar on and before 1582-10-05, and the Gregorian calendar
        after 1582-10-15; if 'proleptic_gregorian', Julian day follows
        the Gregorian calendar; if 'julian', Julian day follows the
        Julian calendar

    Returns
    -------

    jd : float
        the Julian day, with resolution of 1 second
    """

    # Based on `redate.py` by David Finlayson.
    year, month, day = date.year, date.month, date.day
    hour, minute, second = date.hour, date.minute, date.second

    # Convert time to fractions of a day.
    day = day + hour / 24.0 + minute / 1440.0 + second / 86400.0

    # Start Meeus algorithm (variables are in his notation).
    if month < 3:
        month = month + 12
        year = year - 1
    A = int(year / 100)
    jd = int(365.25 * (year + 4716)) + int(30.6001 * (month + 1)) + day - 1524.5

    # Optionally adjust `jd` for the switch from Julian to Gregorian calendar.
    # Here assumed to have occurred the day after 1582 October 4
    if calendar in ["standard", "gregorian"]:
        if jd >= 2299170.5:
            # 1582 October 15 (Gregorian Calendar).
            B = 2 - A + int(A / 4)
        elif jd < 2299160.5:
            # 1582 October 5 (Julian Calendar).
            B = 0
        else:
            raise ValueError("impossible date (falls in gap between end of "
                             "Julian calendar and start of Gregorian calendar")
    elif calendar == "proleptic_gregorian":
        B = 2 - A + int(A / 4)
    elif calendar == "julian":
        B = 0
    else:
        raise ValueError("unknown calendar '{0}' (must be one of 'standard', "
                         "'gregorian', 'proleptic_gregorian' or 'julian'".format(calendar))

    # Adjust for Julian calendar if necessary.
    jd = jd + B

    return jd


def epem(date):
    """Return the Greenwich hour angle.

    Parameters
    ----------

    date : datetime.datetime
        a :class:`datetime.datetime` object (assumed UTC)

    Returns
    -------

    gha : float
        Greenwich hour angle, i.e. the angle between the Greenwich
        meridian and the meridian containing the subsolar point.

    dec : float
        solar declination
    """

    dg2rad = np.pi / 180.
    rad2dg = 1. / dg2rad

    # Compute Julian day from UTC `datetime.datetime` object (note that
    # `datetime.datetime` objects use proleptic Gregorian calendar).
    jday = JulianDayFromDate(date, calendar="proleptic_gregorian")
    jd = np.floor(jday)  # Truncate to integer.

    # Compute UTC hour.
    ut = date.hour + date.minute / 60. + date.second / 3600.

    # Calculate number of centuries from J2000.
    t = (jd + (ut / 24.) - 2451545.0) / 36525.

    # Mean longitude corrected for aberration.
    l = (280.460 + 36000.770 * t) % 360  # noqa: E741

    # Mean anomaly.
    g = 357.528 + 35999.050 * t

    # Ecliptic longitude.
    lm = l + 1.915 * np.sin(g * dg2rad) + 0.020 * np.sin(2 * g * dg2rad)

    # Obliquity of the ecliptic.
    ep = 23.4393 - 0.01300 * t

    # Equation of time.
    eqtime = (-1.915 * np.sin(g * dg2rad) - 0.020 * np.sin(2 * g * dg2rad) +
              +2.466 * np.sin(2 * lm * dg2rad) - 0.053 * np.sin(4 * lm * dg2rad))

    # Greenwich hour angle.
    gha = 15 * ut - 180 + eqtime

    # Declination of Sun.
    dec = np.arcsin(np.sin(ep * dg2rad) * np.sin(lm * dg2rad)) * rad2dg

    return gha, dec


def daynight_terminator(date, delta, lonmin, lonmax):
    """Return the day-night terminator.

    Parameters
    ----------

    date : datetime.datetime
        a :class:`datetime.datetime` object (assumed UTC)

    delta : float
        input longitude grid step in degrees used to compute the
        day-night terminator

    lonmin : float
        minimum input longitude in degrees

    lonmax : float
        maximum input longitude in degrees

    Returns
    -------

    lons : array-like
        array of longitudes of the day-night terminator

    lats : array-like
        array of latitudes of the day-night terminator

    tau : float
        Greenwich hour angle for the input datetime

    dec : float
        solar declination for the input datetime
    """

    dg2rad = np.pi / 180.
    lons = np.arange(lonmin, lonmax + 0.5 * delta, delta, dtype=np.float32)

    # Compute Greenwich hour angle and solar declination.
    tau, dec = epem(date)

    # compute day-night terminator from Greenwich hour angle and declination.
    longitude = lons + tau
    lats = np.arctan(-np.cos(longitude * dg2rad) / np.tan(dec * dg2rad)) / dg2rad

    return lons, lats, tau, dec


def daynight_grid(date, delta, lonmin, lonmax):
    """Return day-night mask array.

    Parameters
    ----------

    date : datetime.datetime
        a :class:`datetime.datetime` object (assumed UTC)

    delta : float
        input longitude grid step in degrees used to compute the
        day-night terminator

    lonmin : float
        minimum input longitude in degrees

    lonmax : float
        maximum input longitude in degrees

    Returns
    -------

    lons2 : array-like
        meshgrid of longitudes of the day-night mask array

    lats2 : array-like
        meshgrid of latitudes of the day-night mask array

    daynight : ~numpy.ma.MaskedArray
        day-night mask array (masked for day, 1 for night)
    """

    lons, lats, _, dec = daynight_terminator(date, delta, lonmin, lonmax)

    # Create meshgrid of longitudes and latitudes.
    lats2 = np.arange(-90, 90 + 0.5 * delta, delta, dtype=np.float32)
    lons2, lats2 = np.meshgrid(lons, lats2)

    # Create day-night grid (0 for day, 1 for night).
    nlats, nlons = len(lats2), len(lons)
    lats = lats[np.newaxis, :] * np.ones((nlats, nlons), dtype=np.float32)
    daynight = np.ones(lons2.shape, np.int8)
    if dec > 0:  # NH summer
        daynight = np.where(lats2 > lats, 0, daynight)
    else:        # NH winter
        daynight = np.where(lats2 < lats, 0, daynight)

    # Create day-night masked array (with day areas masked).
    daynight_mask = 1 - daynight
    daynight = np.ma.array(daynight, mask=daynight_mask)

    return lons2, lats2, daynight



from __future__ import (absolute_import, division, print_function)

# make plots of etopo bathymetry/topography data on
# various map projections, drawing coastlines, state and
# country boundaries, filling continents and drawing
# parallels/meridians

# illustrates special-case polar-centric projections.

from mpl_toolkits.basemap import Basemap, cm
import numpy as np
import matplotlib.pyplot as plt

# read in topo data (on a regular lat/lon grid)
# longitudes go from 20 to 380.
etopo = np.loadtxt('etopo20data.gz')
lons = np.loadtxt('etopo20lons.gz')
lats = np.loadtxt('etopo20lats.gz')

print('min/max etopo20 data:')
print(etopo.min(),etopo.max())

# these are the 4 polar projections
projs = ['laea','stere','aeqd','ortho'] # short names
# long names
projnames = ['Lambert Azimuthal Equal Area','Stereographic','Azimuthal Equidistant','Orthographic']
# loop over hemispheres, make a 4-panel plot for each hemisphere
# showing all four polar projections.
for hem in ['North','South']:
    if hem == 'South':
        lon_0 = -130.
        lon_0_ortho = lon_0 - 180.
        lat_0 = -90.
        #  Lambert Azimuth bounding lat must not extend into opposite hem.
        bounding_lat = -0.01
    elif hem == 'North':
        lon_0 = 130.
        lon_0_ortho = lon_0
        lat_0 = 90.
        #  Lambert Azimuth bounding lat must not extend into opposite hem.
        bounding_lat = 0.01
    # loop over projections, one for each panel of the figure.
    fig = plt.figure(figsize=(8,8))
    npanel = 0
    for proj,projname in zip(projs,projnames):
        npanel = npanel + 1
        if hem == 'South':
            projection = 'sp'+proj
        elif hem == 'North':
            projection = 'np'+proj
        # setup map projection
        # centered on Australia (for SH) or US (for NH).
        if proj == 'ortho':
           m = Basemap(projection='ortho',
                       resolution='c',area_thresh=10000.,lat_0=lat_0,lon_0=lon_0_ortho)
        else:
           m = Basemap(boundinglat=bounding_lat,lon_0=lon_0,\
                       resolution='c',area_thresh=10000.,projection=projection,round=True)
        # compute native map projection coordinates for lat/lon grid.
        x,y = m(*np.meshgrid(lons,lats))
        ax = fig.add_subplot(2,2,npanel)
        # make filled contour plot.
        cs = m.contourf(x,y,etopo,np.linspace(-7500,4500,41),cmap=cm.GMT_haxby)
        # draw coastlines.
        m.drawcoastlines()
        # draw parallels and meridians.
        m.drawparallels(np.arange(-80.,90,20.))
        #labels = [l,r,t,b]
        m.drawmeridians(np.arange(0.,340.,30.),labels=[1,1,1,1],fontsize=7)
        # draw boundary around map region.
        m.drawmapboundary()
        # draw title.
        plt.title(hem+' Polar '+projname,y=1.05,fontsize=12)
        print('plotting '+hem+' Polar '+projname+' basemap ...')
plt.show()



from __future__ import (absolute_import, division, print_function)

# example showing how to use Line Integral Convolution to visualize a vector
# flow field (from Hurricane Earl).  Produces something akin to streamlines.
# Requires vectorplot scikit (http://scikits.appspot.com/vectorplot).
from netCDF4 import Dataset as NetCDFFile
from mpl_toolkits.basemap import Basemap, interp
import numpy as np
import matplotlib.pyplot as plt
try:
    from scikits.vectorplot import lic_internal
except ImportError:
    raise ImportError('need vectorplot scikit for this example')

# H*wind data from http://www.aoml.noaa.gov/hrd/data_sub/wind.html
ncfile = NetCDFFile('rita.nc')
udat = ncfile.variables['sfc_u'][0,:,:]
vdat = ncfile.variables['sfc_v'][0,:,:]
lons1 = ncfile.variables['longitude'][:]
lats1 = ncfile.variables['latitude'][:]
lat0 = lats1[len(lats1)/2]; lon0 = lons1[len(lons1)/2]
lons, lats = np.meshgrid(lons1,lats1)
ncfile.close()

# downsample to finer grid for nicer looking plot.
nlats = 2*udat.shape[0]; nlons = 2*udat.shape[1]
lons = np.linspace(lons1[0],lons1[-1],nlons)
lats = np.linspace(lats1[0],lats1[-1],nlats)
lons, lats = np.meshgrid(lons, lats)
udat = interp(udat,lons1,lats1,lons,lats,order=3)
vdat = interp(vdat,lons1,lats1,lons,lats,order=3)


fig = plt.figure(figsize=(8,8))
m = Basemap(projection='cyl',llcrnrlat=lats1[0],llcrnrlon=lons1[0],urcrnrlat=lats1[-1],urcrnrlon=lons1[-1],resolution='i')
# pass texture, kernel and data to LIC function from vectorplot.
kernellen=31
texture = np.random.rand(udat.shape[0],udat.shape[1]).astype(np.float32)
kernel = np.sin(np.arange(kernellen)*np.pi/kernellen).astype(np.float32)
image = lic_internal.line_integral_convolution(udat.astype(np.float32),\
        vdat.astype(np.float32), texture, kernel)
# plot the resulting image.
im = m.imshow(image,plt.cm.gist_stern)
m.drawcoastlines()
m.drawmeridians(np.arange(-120,-60,2),labels=[0,0,0,1])
m.drawparallels(np.arange(0,30,2),labels=[1,0,0,0])
plt.title('Hurricane Rita flow field visualized with Line Integral Convolution',\
        fontsize=13)
plt.show()



from __future__ import (absolute_import, division, print_function)

from mpl_toolkits.basemap import Basemap
import numpy as np
import matplotlib.pyplot as plt
import sys

def get_input(prompt):
    if sys.hexversion > 0x03000000:
        return input(prompt)
    else:
        return raw_input(prompt)

# create Basemap instance for Geostationary (satellite view) projection.
lon_0 = float(get_input('enter reference longitude (lon_0):'))

# map with land/sea mask plotted
fig=plt.figure()
m = Basemap(projection='geos',lon_0=lon_0,rsphere=(6378137.00,6356752.3142),resolution=None)
# plot land-sea mask.
# land red, oceans blue.
# lakes=True means plot inland lakes with ocean color.
m.drawmapboundary()
m.drawlsmask(land_color='red',ocean_color='blue',lakes=True)
# draw parallels and meridians.
m.drawparallels(np.arange(-90.,120.,30.))
m.drawmeridians(np.arange(0.,420.,60.))
plt.title('Geostationary Map Centered on Lon=%s' % (lon_0))

# map with continents drawn and filled.
fig = plt.figure()
m = Basemap(projection='geos',lon_0=lon_0,rsphere=(6378137.00,6356752.3142),resolution='l')
m.drawcoastlines()
m.fillcontinents(color='coral',lake_color='aqua')
m.drawmapboundary(fill_color='aqua')
m.drawcountries()
# draw parallels and meridians.
m.drawparallels(np.arange(-90.,120.,30.))
m.drawmeridians(np.arange(0.,420.,60.))
plt.title('Geostationary Map Centered on Lon=%s' % (lon_0))
plt.show()



from __future__ import (absolute_import, division, print_function)

from __future__ import unicode_literals
from mpl_toolkits.basemap import Basemap
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import FuncFormatter

# example showing how to create custom tick labels for a cylindrical
# projection.

def lat2str(deg):
    min = 60 * (deg - np.floor(deg))
    deg = np.floor(deg)
    dir = 'N'
    if deg < 0:
        if min != 0.0:
            deg += 1.0
            min -= 60.0
        dir = 'S'
    return ("%d\N{DEGREE SIGN} %g' %s") % (np.abs(deg),np.abs(min),dir)

def lon2str(deg):
    min = 60 * (deg - np.floor(deg))
    deg = np.floor(deg)
    dir = 'E'
    if deg < 0:
        if min != 0.0:
            deg += 1.0
            min -= 60.0
        dir = 'W'
    return ("%d\N{DEGREE SIGN} %g' %s") % (np.abs(deg),np.abs(min),dir)

# (1) use matplotlib custom tick formatter
#     instead of Basemap labelling methods.

# create figure.
fig=plt.figure()
# create Basemap instance (regular lat/lon projection).
# suppress_ticks=False allows custom axes ticks to be used
# Ticks are suppressed by default, so Basemap methods
# drawparallels and drawmeridians used to draw labelled lat/lon grid.
m = Basemap(llcrnrlon=-156.5,llcrnrlat=18.75,urcrnrlon=-154.5,urcrnrlat=20.5,
            resolution='h',projection='cyl',suppress_ticks=False)
# draw coastlines, fill land and lake areas.
m.drawcoastlines()
m.fillcontinents(color='coral',lake_color='aqua')
# background color will be used for oceans.
m.drawmapboundary(fill_color='aqua')
# get axes instance.
ax = plt.gca()
# add custom ticks.
# This only works for projection='cyl'.
def xformat(x, pos=None): return lon2str(x)
xformatter = FuncFormatter(xformat)
ax.xaxis.set_major_formatter(xformatter)
def yformat(y, pos=None): return lat2str(y)
yformatter = FuncFormatter(yformat)
ax.yaxis.set_major_formatter(yformatter)
ax.fmt_xdata = lambda x: lon2str(x)
ax.fmt_ydata = lambda y: lat2str(y)
ax.grid()
ax.set_title('Hawaii')

# (2) use Basemap labelling methods, but pass a
#     custom formatting function with the 'fmt' keyword.

# create figure.
fig = plt.figure()
# create Basemap instance.
m = Basemap(llcrnrlon=-156.5,llcrnrlat=18.75,urcrnrlon=-154.5,urcrnrlat=20.5,
            resolution='h',projection='cyl')
# draw coastlines, fill land and lake areas.
m.drawcoastlines()
m.fillcontinents(color='coral',lake_color='aqua')
# background color will be used for oceans.
m.drawmapboundary(fill_color='aqua')
# label meridians and parallels, passing string formatting function
# with 'fmt' keyword.
m.drawparallels(np.linspace(18,21,7),labels=[1,0,0,0],fmt=lat2str,dashes=[2,2])
m.drawmeridians(np.linspace(-157,-154,7),labels=[0,0,0,1],fmt=lon2str,dashes=[2,2])
plt.title('Hawaii')

plt.show()



from __future__ import (absolute_import, division, print_function)

#!/usr/bin/env python
"""
An example of how to use wx or wxagg in an application with the Basemap module
"""

from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wx import NavigationToolbar2Wx
from matplotlib.figure import Figure

from mpl_toolkits.basemap import Basemap

from wx import *

class CanvasFrame(Frame):

   def __init__(self):
       Frame.__init__(self,None,-1,
                                       'CanvasFrame',size=(550,350))

       self.SetBackgroundColour(NamedColor("WHITE"))

       self.figure = Figure()
       
       self.canvas = FigureCanvas(self, -1, self.figure)
       self.ax = self.figure.add_subplot(111)

       self.sizer = BoxSizer(VERTICAL)
       self.sizer.Add(self.canvas, 1, LEFT | TOP | GROW)
       self.SetSizer(self.sizer)
       self.Fit()

       self.add_toolbar()  # comment this out for no toolbar
       
       self.plot_map()
       
   def add_toolbar(self):
       self.toolbar = NavigationToolbar2Wx(self.canvas)
       self.toolbar.Realize()
       if Platform == '__WXMAC__':
          # Mac platform (OSX 10.3, MacPython) does not seem to cope with
          # having a toolbar in a sizer. This work-around gets the buttons
          # back, but at the expense of having the toolbar at the top
          self.SetToolBar(self.toolbar)
       else:
          # On Windows platform, default window size is incorrect, so set
          # toolbar width to figure width.
          tw, th = self.toolbar.GetSizeTuple()
          fw, fh = self.canvas.GetSizeTuple()
          # By adding toolbar in sizer, we are able to put it at the bottom
          # of the frame - so appearance is closer to GTK version.
          # As noted above, doesn't work for Mac.
          self.toolbar.SetSize(Size(fw, th))
          self.sizer.Add(self.toolbar, 0, LEFT | EXPAND)
       # update the axes menu on the toolbar
       self.toolbar.update()
           
   def plot_map(self):
       map = Basemap(ax=self.ax)
       map.drawcoastlines()
       map.drawcountries()
       map.drawmapboundary()
       map.fillcontinents(color='coral', lake_color='aqua')
       map.drawmapboundary(fill_color='aqua')
       self.figure.canvas.draw()

class App(App):

   def OnInit(self):
       'Create the main window and insert the custom frame'
       frame = CanvasFrame()
       frame.Show(True)
       return True

app = App(0)
app.MainLoop()



from __future__ import (absolute_import, division, print_function)

from mpl_toolkits.basemap import Basemap
import numpy as np
import matplotlib.pyplot as plt

# read in data.
file = open('fcover.dat','r')
ul=[];vl=[];pl=[]
nlons=73; nlats=73
dellat = 2.5; dellon = 5.
for line in file.readlines():
   l = line.replace('\n','').split()
   ul.append(float(l[0]))
   vl.append(float(l[1]))
   pl.append(float(l[2]))
u = np.reshape(np.array(ul,np.float32),(nlats,nlons))
v = np.reshape(np.array(vl,np.float32),(nlats,nlons))
p = np.reshape(np.array(pl,np.float32),(nlats,nlons))
lats1 = -90.+dellat*np.arange(nlats)
lons1 = -180.+dellon*np.arange(nlons)
lons, lats = np.meshgrid(lons1, lats1)
# convert from mps to knots.
u = 1.944*u; v = 1.944*v

# plot barbs in map projection coordinates.

# stereogrpaphic projection.
m = Basemap(width=10000000,height=10000000,lon_0=-90,lat_0=45.,lat_ts=45,
            resolution='l',projection='stere')
x,y = m(lons,lats)
# transform from spherical to map projection coordinates (rotation
# and interpolation).
nxv = 25; nyv = 25
udat, vdat, xv, yv = m.transform_vector(u,v,lons1,lats1,nxv,nyv,returnxy=True)
# create a figure, add an axes.
fig=plt.figure(figsize=(8,6))
ax = fig.add_axes([0.1,0.1,0.8,0.8])
# plot color-filled contours over map
levs = np.arange(960,1051,4)
cs1 = m.contour(x,y,p,levs,colors='k',linewidths=0.5)
cs2 = m.contourf(x,y,p,levs)
# plot barbs.
m.barbs(xv,yv,udat,vdat,length=6,barbcolor='k',flagcolor='r',linewidth=0.5)
# plot colorbar for pressure
m.colorbar(pad='12%') # draw colorbar
# draw coastlines
m.drawcoastlines()
# draw parallels
m.drawparallels(np.arange(0,81,20),labels=[1,1,0,0])
# draw meridians
m.drawmeridians(np.arange(-180,0,20),labels=[0,0,0,1])
plt.title('Surface Wind Barbs and Pressure (NH)')

# stereogrpaphic projection (SH).
# 'flip_barb' flag is automatically set for SH data, so that
# barbs point toward lower pressure (in both Hemisphere).
m = Basemap(width=10000000,height=10000000,lon_0=-90,lat_0=-45.,lat_ts=-45,
            resolution='l',projection='stere')
x,y = m(lons,lats)
# transform from spherical to map projection coordinates (rotation
# and interpolation).
nxv = 25; nyv = 25
udat, vdat, xv, yv = m.transform_vector(u,v,lons1,lats1,nxv,nyv,returnxy=True)
# create a figure, add an axes.
fig=plt.figure(figsize=(8,6))
ax = fig.add_axes([0.1,0.1,0.8,0.8])
# plot color-filled contours over map
levs = np.arange(960,1051,4)
cs1 = m.contour(x,y,p,levs,colors='k',linewidths=0.5)
cs2 = m.contourf(x,y,p,levs)
# plot barbs.
m.barbs(xv,yv,udat,vdat,length=6,barbcolor='k',flagcolor='r',linewidth=0.5)
# plot colorbar for pressure
m.colorbar(pad='12%') # draw colorbar
# draw coastlines
m.drawcoastlines()
# draw parallels
m.drawparallels(np.arange(-80,-19,20),labels=[1,1,0,0])
# draw meridians
m.drawmeridians(np.arange(-180,0,20),labels=[0,0,1,0])
plt.title('Surface Wind Barbs and Pressure (SH)',y=1.04)
plt.show()



#!/usr/bin/env python
from __future__ import print_function
import fileinput
import glob
import os
import shutil
import sys

def html():
    os.system('sphinx-build -b html -d build/doctrees . build/html')

def latex():
    if sys.platform != 'win32':
        # LaTeX format.
        os.system('sphinx-build -b latex -d build/doctrees . build/latex')

        # Produce pdf.
        os.chdir('build/latex')

        # Copying the makefile produced by sphinx...
        os.system('pdflatex Basemap.tex')
        os.system('pdflatex Basemap.tex')
        os.system('makeindex -s python.ist Basemap.idx')
        os.system('makeindex -s python.ist modBasemap.idx')
        os.system('pdflatex Basemap.tex')

        os.chdir('../..')
    else:
        print('latex build has not been tested on windows')

def clean():
    shutil.rmtree('build')

def all():
    html()
    latex()


funcd = {
         'html':html,
         'latex':latex,
         'clean':clean,
         'all':all,
         }


if len(sys.argv)>1:
    for arg in sys.argv[1:]:
        func = funcd.get(arg)
        if func is None:
            raise SystemExit('Do not know how to handle %s; valid args are'%(
                    arg, list(funcd.keys())))
        func()
else:
    all()



"""Import test for the :mod:`mpl_toolkits.basemap.Basemap` class."""

import os
import shutil
import tempfile
import datetime as dt
import unittest

import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.collections import LineCollection
from matplotlib.contour import QuadContourSet
from matplotlib.image import AxesImage
from matplotlib.patches import Polygon
from mpl_toolkits.basemap import Basemap
from mpl_toolkits.basemap import shiftgrid

try:
    import PIL
except ImportError:
    PIL = None


mpl_version = tuple(map(int, mpl.__version__.split(".")[:2]))


class TestMplToolkitsBasemapBasemap(unittest.TestCase):
    """Unittest class for the :mod:`mpl_toolkits.basemap.Basemap` class."""

    def setUp(self):
        """Define the setup of test scope variables."""

    def tearDown(self):
        """Define the teardown of test scope variables."""

        axs_obj = plt.gca()
        axs_obj.clear()

    def test_init_with_ortho(self, resolution=None):
        """Test init with orthographic projection and a given resolution."""

        bmap = Basemap(projection="ortho", resolution=resolution,
                       lat_1=45, lat_2=55, lat_0=50, lon_0=-107)
        self.assertIsInstance(bmap, Basemap)

    def test_init_with_ortho_c_resolution(self):
        """Test init with orthographic projection and a given resolution."""

        self.test_init_with_ortho(resolution="c")

    def test_init_with_ortho_l_resolution(self):
        """Test init with orthographic projection and a given resolution."""

        self.test_init_with_ortho(resolution="l")

    def test_init_with_ortho_i_resolution(self):
        """Test init with orthographic projection and a given resolution."""

        self.test_init_with_ortho(resolution="i")

    def test_init_with_optional_casting(self):
        """Test init for the bug reported in GitHub issue #260."""

        kwds = {
            "llcrnrlat":
                28.979408,
            "urcrnrlat":
                35.19622,
            "llcrnrlon":
                -95.614105,
            "urcrnrlon":
                -77.554749,
            "lon_0":
                -87.0,
            "resolution":
                "c",
            "lat_0":
                32.070374,
            "projection":
                "lcc"
        }

        bmap1_lat_1 = 30.0
        bmap2_lat_1 = np.array([30.0], dtype=np.float32)

        bmap1 = Basemap(lat_1=bmap1_lat_1, **kwds)
        bmap2 = Basemap(lat_1=bmap2_lat_1, **kwds)
        self.assertEqual(bmap1.proj4string, bmap2.proj4string)

    def test_drawcoastlines(self, axs=None, axslen0=10):
        """Test that no lines are missing when drawing coastlines."""

        axs_obj = plt.gca() if axs is None else axs
        axs_children = axs_obj.get_children()
        self.assertEqual(len(axs_children), axslen0)

        bmap = Basemap(projection="merc", resolution="i", lat_ts=20,
                       llcrnrlat=36.0, llcrnrlon=6.0,
                       urcrnrlat=47.7, urcrnrlon=19.0)

        collection = bmap.drawcoastlines(linewidth=1, color="red")
        self.assertIsInstance(collection, LineCollection)

        axs_children = axs_obj.get_children()
        self.assertEqual(len(axs_children), axslen0 + 1)

        lines = collection.get_paths()
        self.assertEqual(len(lines), 27)

    def test_drawcountries(self, axs=None, axslen0=10):
        """Test that no lines are missing when drawing country boundaries."""

        axs_obj = plt.gca() if axs is None else axs
        axs_children = axs_obj.get_children()
        self.assertEqual(len(axs_children), axslen0)

        bmap = Basemap(projection="merc", resolution="i", lat_ts=20,
                       llcrnrlat=36.0, llcrnrlon=6.0,
                       urcrnrlat=47.7, urcrnrlon=19.0)

        collection = bmap.drawcountries(linewidth=1, color="blue")
        self.assertIsInstance(collection, LineCollection)

        axs_children = axs_obj.get_children()
        self.assertEqual(len(axs_children), axslen0 + 1)

        lines = collection.get_paths()
        self.assertEqual(len(lines), 29)

    @unittest.skipIf(PIL is None, reason="pillow unavailable")
    def test_arcgisimage_with_cyl(self, axs=None, axslen0=10):
        """Test showing an ArcGIS image as background."""

        axs_obj = plt.gca() if axs is None else axs
        axs_children = axs_obj.get_children()
        self.assertEqual(len(axs_children), axslen0)

        bmap = Basemap(ax=axs, projection="cyl", resolution=None,
                       llcrnrlon=-90, llcrnrlat=30,
                       urcrnrlon=-60, urcrnrlat=60)
        img = bmap.arcgisimage(verbose=False)
        self.assertIsInstance(img, AxesImage)

        axs_children = axs_obj.get_children()
        self.assertEqual(len(axs_children), axslen0 + 1)

    @unittest.skipIf(PIL is None, reason="pillow unavailable")
    def test_arcgisimage_with_cyl_using_cache(self, existing=False, axs=None, axslen0=10):
        """Test showing an ArcGIS image as background."""

        axs_obj = plt.gca() if axs is None else axs
        axs_children = axs_obj.get_children()
        self.assertEqual(len(axs_children), axslen0)

        bmap = Basemap(ax=axs, projection="cyl", resolution=None,
                       llcrnrlon=-90, llcrnrlat=30,
                       urcrnrlon=-60, urcrnrlat=60)

        # Create cache directory string and check it is empty.
        tmpdir = tempfile.mkdtemp(prefix="tmp-basemap-cachedir-")
        cachedir = tmpdir if existing else os.path.join(tmpdir, "cachedir")
        if os.path.isdir(cachedir):
            self.assertEqual(len(os.listdir(cachedir)), 0)

        try:
            # Check that the first call populates the cache.
            img = bmap.arcgisimage(verbose=False, cachedir=cachedir)
            self.assertEqual(len(os.listdir(cachedir)), 1)
            # Check output properties after the first call.
            self.assertIsInstance(img, AxesImage)
            axs_children = axs_obj.get_children()
            self.assertEqual(len(axs_children), axslen0 + 1)
            # Check that the second call does not update the cache.
            img = bmap.arcgisimage(verbose=False, cachedir=cachedir)
            self.assertEqual(len(os.listdir(cachedir)), 1)
            # Check output properties after the second call.
            self.assertIsInstance(img, AxesImage)
            axs_children = axs_obj.get_children()
            self.assertEqual(len(axs_children), axslen0 + 2)
        finally:
            if os.path.isdir(tmpdir):
                shutil.rmtree(tmpdir)

    @unittest.skipIf(PIL is None, reason="pillow unavailable")
    def test_arcgisimage_with_cyl_using_cache_already_existing(self):
        """Test showing an ArcGIS image as background."""

        self.test_arcgisimage_with_cyl_using_cache(existing=True)

    def _test_basemap_data_warpimage(self, method, axs=None, axslen0=10):
        """Test drawing a map background from :mod:`basemap_data`."""

        axs_obj = plt.gca() if axs is None else axs
        axs_children = axs_obj.get_children()
        self.assertEqual(len(axs_children), axslen0)

        bmap = Basemap(ax=axs, projection="moll", resolution=None, lon_0=0)
        img = getattr(bmap, method)(ax=axs, scale=0.1)
        self.assertIsInstance(img, AxesImage)

        flag = int(mpl_version < (3, 5))
        axs_children = axs_obj.get_children()
        self.assertEqual(len(axs_children), axslen0 + 3)
        self.assertIsInstance(axs_children[1 - flag], Polygon)
        self.assertIsInstance(axs_children[2 - flag], Polygon)
        self.assertIsInstance(axs_children[(axslen0 + 1) * flag], AxesImage)
        self.assertIs(axs_children[(axslen0 + 1) * flag], img)

    @unittest.skipIf(PIL is None, reason="pillow unavailable")
    def test_bluemarble(self, axs=None, axslen0=10):
        """Test drawing a map with a blue marble image as background."""

        method = "bluemarble"
        self._test_basemap_data_warpimage(method, axs=axs, axslen0=axslen0)

    @unittest.skipIf(PIL is None, reason="pillow unavailable")
    def test_bluemarble_with_custom_axes(self):
        """Test drawing a map with a blue marble image as background."""

        _, axs = plt.subplots()
        self.test_bluemarble(axs=axs, axslen0=10)

    @unittest.skipIf(PIL is None, reason="pillow unavailable")
    def test_etopo(self, axs=None, axslen0=10):
        """Test drawing a map with an ETOPO relief image as background."""

        method = "etopo"
        self._test_basemap_data_warpimage(method, axs=axs, axslen0=axslen0)

    @unittest.skipIf(PIL is None, reason="pillow unavailable")
    def test_etopo_with_custom_axes(self):
        """Test drawing a map with an ETOPO relief image as background."""

        _, axs = plt.subplots()
        self.test_etopo(axs=axs, axslen0=10)

    @unittest.skipIf(PIL is None, reason="pillow unavailable")
    def test_shadedrelief(self, axs=None, axslen0=10):
        """Test drawing a map with a shaded relief image as background."""

        method = "shadedrelief"
        self._test_basemap_data_warpimage(method, axs=axs, axslen0=axslen0)

    @unittest.skipIf(PIL is None, reason="pillow unavailable")
    def test_shadedrelief_with_custom_axes(self):
        """Test drawing a map with a shaded relief image as background."""

        _, axs = plt.subplots()
        self.test_shadedrelief(axs=axs, axslen0=10)

    def _test_generic_contour_function(self, function):
        """Generic test for the `contour` and `contourf` methods."""

        bmap = Basemap(projection="ortho", lat_0=45, lon_0=-100, resolution=None)

        # Create a regular lat/lon grid.
        nlats = 73
        nlons = 145
        delta = 2 * np.pi / (nlons - 1)
        indx = np.indices((nlats, nlons))
        lats = (0.5 * np.pi - delta * indx[0, :, :])
        lons = (delta * indx[1, :, :])

        # Create some data the regular lat/lon grid.
        mean = 0.50 * np.cos(2 * lats) * ((np.sin(2 * lats))**2 + 2)
        wave = 0.75 * np.cos(4 * lons) * np.sin(2 * lats)**8
        data = mean + wave

        # Compute native map projection coordinates of lat/lon grid.
        x, y = bmap(np.degrees(lons), np.degrees(lats))

        # Contour data over the map and check output.
        cset = getattr(bmap, function)(x, y, data, 15)
        self.assertIsInstance(cset, QuadContourSet)

    def test_contour(self):
        """Test drawing contours on a map."""

        self._test_generic_contour_function("contour")

    def test_contourf(self):
        """Test drawing filled contours on a map."""

        self._test_generic_contour_function("contourf")

    def test_nightshade(self):
        """Test drawing the day/night terminator and night shade on a map."""

        bmap = Basemap(projection="mill", lon_0=180)
        cset = bmap.nightshade(date=dt.datetime(1970, 1, 1))
        self.assertIsInstance(cset, QuadContourSet)


class TestMplToolkitsBasemapBasemapCall(unittest.TestCase):
    """Unittest class for :meth:`mpl_toolkits.basemap.Basemap.__call__`."""

    def setUp(self):
        """Define the setup of test scope variables."""

    def tearDown(self):
        """Define the teardown of test scope variables."""

        axs_obj = plt.gca()
        axs_obj.clear()

    @staticmethod
    def get_input_objects():
        """Return geographic coordinate arrays and :class:`Basemap` instance."""

        lons, lats = np.arange(-180, 180, 20), np.arange(-90, 90, 10)
        lats, lons = np.meshgrid(lats, lons)
        lons, lats = lons.copy(order="F"), lats.copy(order="F")
        return Basemap(projection="sinu", lon_0=0), lons, lats

    def test_transform_with_c_non_contiguous_arrays(self):
        """Test transform with C non-contiguous arrays."""

        bmap, lons, lats = self.get_input_objects()
        self.assertIsInstance(bmap, Basemap)
        self.assertIsInstance(lats, np.ndarray)
        self.assertIsInstance(lons, np.ndarray)
        self.assertFalse(lats.flags["C_CONTIGUOUS"])
        self.assertFalse(lons.flags["C_CONTIGUOUS"])

        xx1, yy1 = bmap(lons, lats)
        self.assertIsInstance(xx1, np.ndarray)
        self.assertIsInstance(yy1, np.ndarray)
        self.assertEqual(xx1.shape, lons.shape)
        self.assertEqual(yy1.shape, lats.shape)

    def test_transform_equal_with_c_and_f_order_arrays(self):
        """Test transform with C contiguous arrays and F contiguous arrays."""

        bmap, lons, lats = self.get_input_objects()

        xx1, yy1 = bmap(lons.copy(order="C"), lats.copy(order="C"))
        xx2, yy2 = bmap(lons.copy(order="F"), lats.copy(order="F"))
        self.assertTrue(np.allclose(xx1, xx2))
        self.assertTrue(np.allclose(yy1, yy2))


class TestMplToolkitsBasemapBasemapShiftData(unittest.TestCase):
    """Unittest class for :meth:`mpl_toolkits.basemap.Basemap.shiftdata`."""

    def setUp(self):
        """Define the setup of test scope variables."""

    def tearDown(self):
        """Define the teardown of test scope variables."""

        axs_obj = plt.gca()
        axs_obj.clear()

    def generate_2d_longitude_grid(self, lons1d):
        """Return a 2D longitude grid."""

        lats = [10] * len(lons1d)
        return np.meshgrid(lons1d, lats)[0]

    def test_shiftdata_with_non_monotonous_longitudes(self):
        """Test that shiftdata works with non-monontonous longitudes.

        For example, this is the case with scatter data.
        """

        # Before, having several break points would cause the exception
        # inside the `shiftdata` method called from `scatter` method.
        lons = [179, 180, 180, 0, 290, 10, 320, -150, 350, -250, 250]
        bmap = Basemap(projection="cyl", resolution=None, lon_0=0)
        lons_new = bmap.shiftdata(lons, fix_wrap_around=True)
        for lon in lons_new:
            self.assertGreaterEqual(lon, bmap.projparams["lon_0"] - 180)
            self.assertLessEqual(lon, bmap.projparams["lon_0"] + 180)

        # Check if the modified longitudes are inside the projection region.
        lons_new = bmap.shiftdata(lons, fix_wrap_around=False)
        for lon in lons_new:
            self.assertGreaterEqual(lon, bmap.projparams["lon_0"] - 180)
            self.assertLessEqual(lon, bmap.projparams["lon_0"] + 180)

    def test_shiftdata_with_monotonous_lons(self):
        """Test that shiftdata works with `fix_wrap_around=True` as before."""

        bmap = Basemap(projection="cyl", resolution=None, lon_0=0)

        lons_in = [120, 140, 160, 180, 200, 220]
        lons_out_expected = [-160, -140, 120, 140, 160, 180]

        lons_out = bmap.shiftdata(lons_in, fix_wrap_around=True)
        self.assertTrue(np.allclose(lons_out, lons_out_expected))

    def test_shiftdata_with_1_point(self):
        """Test that shiftdata works with 1 points."""

        bmap = Basemap(projection="mill", resolution=None,
                       llcrnrlon=0, llcrnrlat=-80,
                       urcrnrlon=360, urcrnrlat=80)

        # This should not fail due to longitude out of bounds.
        lons_out = bmap.shiftdata([361])
        self.assertTrue(np.allclose(lons_out, [1.0]))

        lons_out = bmap.shiftdata([10])
        self.assertTrue(np.allclose(lons_out, [10.0]))

        lons_in = np.asarray([[361.0]])
        lons_out = bmap.shiftdata(lons_in[:])
        self.assertTrue(np.allclose(lons_out, [[1.0]]))

    def test_shiftdata_with_2_points(self):
        """Test that shiftdata works with 2 points."""

        bmap = Basemap(projection="mill", resolution=None,
                       llcrnrlon=0, llcrnrlat=-80,
                       urcrnrlon=360, urcrnrlat=80)

        lons_out_expected = [10, 15, 20]
        lons_out = bmap.shiftdata(lons_out_expected[:])
        self.assertTrue(np.allclose(lons_out, lons_out_expected))

        lons_out_expected = bmap.shiftdata([10, 361, 362])
        lons_out = bmap.shiftdata([10, 361])
        self.assertTrue(np.allclose(lons_out, lons_out_expected[:lons_out.size]))

    def test_shiftdata_with_less_than_n_by_3_points(self):
        """Test that shiftdata works with `(n x 3)` and `(n x 2)` grids."""

        bmap = Basemap(projection="mill", resolution=None,
                       llcrnrlon=0, llcrnrlat=-80,
                       urcrnrlon=360, urcrnrlat=80)

        # Test that nothing should change here.
        lons_out_expected = self.generate_2d_longitude_grid([10, 15, 20])
        lons_out = bmap.shiftdata(lons_out_expected)
        self.assertTrue(np.allclose(lons_out, lons_out_expected))

        # Shift n x 3 and n x 2 grids and compare results in overlapping area.
        lons_in = self.generate_2d_longitude_grid([10, 361, 362])
        lons_out = bmap.shiftdata(lons_in[:, :2])
        lons_out_expected = bmap.shiftdata(lons_in)[:, :2]
        self.assertTrue(np.allclose(lons_out, lons_out_expected))


class TestMplToolkitsBasemapBasemapRotateVector(unittest.TestCase):
    """Unittest class for :meth:`mpl_toolkits.basemap.Basemap.rotate_vector`."""

    def setUp(self):
        """Define the setup of test scope variables."""

    def tearDown(self):
        """Define the teardown of test scope variables."""

        axs_obj = plt.gca()
        axs_obj.clear()

    @staticmethod
    def get_input_objects():
        """Return geographic coordinates and vector components."""

        lat = np.array([0, 45, 75, 90])
        lon = np.array([0, 90, 180, 270])

        shape = (lat.size, lon.size)
        u = np.ones(shape)
        v = np.zeros(shape)

        return u, v, lat, lon

    def test_rotate_vector_cylindrical(self):
        """Test vector rotation with cylindrical projection."""

        u, v, lat, lon = self.get_input_objects()

        bmap = Basemap(projection="cyl", resolution=None)

        # Check that the vectors are identical after rotation.
        # pylint: disable=unbalanced-tuple-unpacking
        rotu, rotv = bmap.rotate_vector(u, v, lon, lat, returnxy=False)
        self.assertTrue(np.allclose(rotu, u))
        self.assertTrue(np.allclose(rotv, v))

    def test_rotate_vector_cylindrical_without_nan(self):
        """Test vector rotation with cylindrical projection."""

        # Set one `u` element to 0, so that the vector magnitude is 0.
        u, v, lat, lon = self.get_input_objects()
        u[1, 1] = 0

        bmap = Basemap(projection="cyl", resolution=None)

        # Check that the vectors are identical after rotation.
        # pylint: disable=unbalanced-tuple-unpacking
        rotu, rotv = bmap.rotate_vector(u, v, lon, lat, returnxy=False)
        self.assertFalse(np.isnan(rotu).any())
        self.assertTrue(np.allclose(rotu, u))
        self.assertTrue(np.allclose(rotv, v))

    def test_rotate_vector_npstere(self):
        """Test vector rotation with NP stereographic projection."""

        # Set all `v` elements to 1.
        u, v, lat, lon = self.get_input_objects()
        v = np.ones(v.shape)

        bmap = Basemap(projection="npstere", resolution=None,
                       boundinglat=50., lon_0=0.)

        # pylint: disable=unbalanced-tuple-unpacking
        rotu, rotv = bmap.rotate_vector(u, v, lon, lat, returnxy=False)
        self.assertTrue(np.allclose(rotu[2, :], [+1, -1, -1, +1]))
        self.assertTrue(np.allclose(rotv[2, :], [+1, +1, -1, -1]))


class TestMplToolkitsBasemapShiftGrid(unittest.TestCase):
    """Unittest class for :func:`mpl_toolkits.basemap.shiftgrid`."""

    def setUp(self):
        """Define the setup of test scope variables."""

    def tearDown(self):
        """Define the teardown of test scope variables."""

    def test_shifgrid_with_cyclic_data(self):
        """Test shiftgrid with some cyclic data."""

        lonin = np.array(
            [0, 30, 60, 90, 120, 150, 180, 210, 240, 270, 300, 330, 360],
            dtype=np.float64)
        gridin = np.array(
            [[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 0]],
            dtype=np.float64)
        lonout = np.array(
            [-180, -150, -120, -90, -60, -30, 0, 30, 60, 90, 120, 150, 180],
            dtype=np.float64)
        gridout = np.array(
            [[6, 7, 8, 9, 10, 11, 0, 1, 2, 3, 4, 5, 6]],
            dtype=np.float64)

        grid, lon = shiftgrid(lonin[len(lonin) // 2], gridin, lonin, start=False)
        self.assertTrue((lon == lonout).all())
        self.assertTrue((grid == gridout).all())

    def test_shiftgrid_with_no_cyclicdata_1(self):
        """Test shiftgrid with some no-cyclic data."""

        lonin = np.array(
            [0, 30, 60, 90, 120, 150, 180, 210, 240, 270, 300, 330],
            dtype=np.float64)
        gridin = np.array(
            [[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]],
            dtype=np.float64)
        lonout = np.array(
            [-180, -150, -120, -90, -60, -30, 0, 30, 60, 90, 120, 150],
            dtype=np.float64)
        gridout = np.array(
            [[6, 7, 8, 9, 10, 11, 0, 1, 2, 3, 4, 5]],
            dtype=np.float64)

        grid, lon = shiftgrid(lonin[len(lonin) // 2], gridin, lonin, start=False)
        self.assertTrue((lon == lonout).all())
        self.assertTrue((grid == gridout).all())

    def test_shiftgrid_with_no_cyclicdata_2(self):
        """Test shiftgrid with some no-cyclic data."""

        lonin = np.array(
            [15, 45, 75, 105, 135, 165, 195, 225, 255, 285, 315, 345],
            dtype=np.float64)
        gridin = np.array(
            [[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]],
            dtype=np.float64)
        lonout = np.array(
            [-165, -135, -105, -75, -45, -15, 15, 45, 75, 105, 135, 165],
            dtype=np.float64)
        gridout = np.array(
            [[6, 7, 8, 9, 10, 11, 0, 1, 2, 3, 4, 5]],
            dtype=np.float64)

        grid, lon = shiftgrid(lonin[len(lonin) // 2], gridin, lonin, start=False)
        self.assertTrue((lon == lonout).all())
        self.assertTrue((grid == gridout).all())


if __name__ == "__main__":
    unittest.main()



import numpy as np
from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
from datetime import datetime
# miller projection 
map = Basemap(projection='mill',lon_0=180)
# plot coastlines, draw label meridians and parallels.
map.drawcoastlines()
map.drawparallels(np.arange(-90,90,30),labels=[1,0,0,0])
map.drawmeridians(np.arange(map.lonmin,map.lonmax+30,60),labels=[0,0,0,1])
# fill continents 'coral' (with zorder=0), color wet areas 'aqua'
map.drawmapboundary(fill_color='aqua')
map.fillcontinents(color='coral',lake_color='aqua')
# shade the night areas, with alpha transparency so the 
# map shows through. Use current time in UTC.
date = datetime.utcnow()
CS=map.nightshade(date)
plt.title('Day/Night Map for %s (UTC)' % date.strftime("%d %b %Y %H:%M:%S"))
plt.show()



from __future__ import (absolute_import, division, print_function)

import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap

def draw_map_background(m, ax):
    ax.set_facecolor('#729FCF')
    m.fillcontinents(color='#FAFAFA', ax=ax, zorder=0)
    m.drawcounties(ax=ax)
    m.drawstates(ax=ax)
    m.drawcountries(ax=ax)
    m.drawcoastlines(ax=ax)
KM = 1000.
clat = 39.3
clon = -94.7333
wid = 5500 * KM
hgt = 3500 * KM
m = Basemap(width=wid, height=hgt, rsphere=(6378137.00,6356752.3142),
            resolution='i', area_thresh=2500., projection='lcc',
            lat_1=38.5, lat_2=38.5, lat_0=clat, lon_0=clon)
fig = plt.figure()
ax = fig.add_subplot(111)
draw_map_background(m, ax)
plt.show()


"""Import test for the :mod:`mpl_toolkits.basemap` package."""

import unittest
from mpl_toolkits import basemap


class TestMplToolkitsBasemap(unittest.TestCase):
    """Unittest class for the :mod:`mpl_toolkits.basemap` package."""

    def test_version_attribute(self):
        """Test that basic library import is working."""

        self.assertTrue(hasattr(basemap, "__version__"))
        basemap_version = basemap.__version__

        num = r"(?:0|[1-9]\d*)"
        build = r"(?:dev[0]?|a[1-4]|b[1-3]|rc[1-2])"
        semver = r"^({0}\.{0}\.{0})(?:[+-\.]?({1}))?$".format(num, build)
        self.assertRegex(basemap_version, semver)


if __name__ == "__main__":
    unittest.main()



from mpl_toolkits.basemap import Basemap, shiftgrid, cm
import numpy as np
import matplotlib.pyplot as plt
from netCDF4 import Dataset

# read in etopo5 topography/bathymetry.
url = 'http://ferret.pmel.noaa.gov/thredds/dodsC/data/PMEL/etopo5.nc'
etopodata = Dataset(url)

topoin = etopodata.variables['ROSE'][:]
lons = etopodata.variables['ETOPO05_X'][:]
lats = etopodata.variables['ETOPO05_Y'][:]
# shift data so lons go from -180 to 180 instead of 20 to 380.
topoin,lons = shiftgrid(180.,topoin,lons,start=False)

# plot topography/bathymetry as an image.

# create the figure and axes instances.
fig = plt.figure()
ax = fig.add_axes([0.1,0.1,0.8,0.8])
# setup of basemap ('lcc' = lambert conformal conic).
# use major and minor sphere radii from WGS84 ellipsoid.
m = Basemap(llcrnrlon=-145.5,llcrnrlat=1.,urcrnrlon=-2.566,urcrnrlat=46.352,\
            rsphere=(6378137.00,6356752.3142),\
            resolution='l',area_thresh=1000.,projection='lcc',\
            lat_1=50.,lon_0=-107.,ax=ax)
# transform to nx x ny regularly spaced 5km native projection grid
nx = int((m.xmax-m.xmin)/5000.)+1; ny = int((m.ymax-m.ymin)/5000.)+1
topodat = m.transform_scalar(topoin,lons,lats,nx,ny)
# plot image over map with imshow.
im = m.imshow(topodat,cm.GMT_haxby)
# draw coastlines and political boundaries.
m.drawcoastlines()
m.drawcountries()
m.drawstates()
# draw parallels and meridians.
# label on left and bottom of map.
parallels = np.arange(0.,80,20.)
m.drawparallels(parallels,labels=[1,0,0,1])
meridians = np.arange(10.,360.,30.)
m.drawmeridians(meridians,labels=[1,0,0,1])
# add colorbar
cb = m.colorbar(im,"right", size="5%", pad='2%')
ax.set_title('ETOPO5 Topography - Lambert Conformal Conic')
plt.show()

# make a shaded relief plot.

# create new figure, axes instance.
fig = plt.figure()
ax = fig.add_axes([0.1,0.1,0.8,0.8])
# attach new axes image to existing Basemap instance.
m.ax = ax
# create light source object.
from matplotlib.colors import LightSource
ls = LightSource(azdeg = 90, altdeg = 20)
# convert data to rgb array including shading from light source.
# (must specify color map)
rgb = ls.shade(topodat, cm.GMT_haxby)
im = m.imshow(rgb)
# draw coastlines and political boundaries.
m.drawcoastlines()
m.drawcountries()
m.drawstates()
ax.set_title('Shaded ETOPO5 Topography - Lambert Conformal Conic')
plt.show()



from __future__ import (absolute_import, division, print_function)

from mpl_toolkits.basemap import Basemap, cm
from netCDF4 import Dataset as NetCDFFile
import numpy as np
import matplotlib.pyplot as plt
import copy
from matplotlib import rcParams

# make tick labels smaller
rcParams['xtick.labelsize']=9
rcParams['ytick.labelsize']=9

# plot rainfall from NWS using special precipitation
# colormap used by the NWS, and included in basemap.

nc = NetCDFFile('nws_precip_conus_20061222.nc')
# data from http://water.weather.gov/precip/
prcpvar = nc.variables['amountofprecip']
data = 0.01*prcpvar[:]
latcorners = nc.variables['lat'][:]
loncorners = -nc.variables['lon'][:]
plottitle = prcpvar.long_name+' for period ending '+prcpvar.dateofdata
print(data.min(), data.max())
print(latcorners)
print(loncorners)
print(plottitle)
print(data.shape)
lon_0 = -nc.variables['true_lon'].getValue()
lat_0 = nc.variables['true_lat'].getValue()
# create polar stereographic Basemap instance.
m = Basemap(projection='stere',lon_0=lon_0,lat_0=90.,lat_ts=lat_0,\
            llcrnrlat=latcorners[0],urcrnrlat=latcorners[2],\
            llcrnrlon=loncorners[0],urcrnrlon=loncorners[2],\
            rsphere=6371200.,resolution='l',area_thresh=10000)
# create figure
fig = plt.figure(figsize=(8.5,11))
plt.subplot(211)
ax = plt.gca()
# draw coastlines, state and country boundaries, edge of map.
m.drawcoastlines()
m.drawstates()
m.drawcountries()
# draw parallels.
delat = 10.0
parallels = np.arange(0.,90,delat)
m.drawparallels(parallels,labels=[1,0,0,0],fontsize=10)
# draw meridians
delon = 10.
meridians = np.arange(180.,360.,delon)
m.drawmeridians(meridians,labels=[0,0,0,1],fontsize=10)
ny = data.shape[0]; nx = data.shape[1]
lons, lats = m.makegrid(nx, ny) # get lat/lons of ny by nx evenly space grid.
x, y = m(lons, lats) # compute map proj coordinates.
# draw filled contours.
clevs = [0,1,2.5,5,7.5,10,15,20,30,40,50,70,100,150,200,250,300,400,500,600,750]
cs = m.contourf(x,y,data,clevs,cmap=cm.s3pcpn)
# draw colorbar.
cbar = m.colorbar(cs,location='bottom',pad="10%")
cbar.set_label('mm')
# plot title
plt.title(plottitle+'- contourf',fontsize=10)

plt.subplot(212)
ax = plt.gca()
# draw coastlines, state and country boundaries, edge of map.
m.drawcoastlines()
m.drawstates()
m.drawcountries()
# draw parallels.
m.drawparallels(parallels,labels=[1,0,0,0],fontsize=10)
# draw meridians
m.drawmeridians(meridians,labels=[0,0,0,1],fontsize=10)
# draw image
im = m.imshow(data,cmap=cm.s3pcpn,interpolation='nearest',vmin=0,vmax=750)
# make a copy of the image object, change
# colormap to linear version of the precip colormap.
im2 = copy.copy(im)
im2.set_cmap(cm.s3pcpn_l)
# draw colorbar using im2, not im (hack to prevent colors from being
# too compressed at the low end on the colorbar - results
# from highly nonuniform colormap)
cb = m.colorbar(im2,location='bottom',pad="10%")
cb.set_label('mm')
# reset colorbar tick labels.
cb.set_ticks(np.linspace(clevs[0],clevs[-1],len(clevs)))
cb.set_ticklabels(['%g' % clev for clev in clevs])
# plot title
plt.title(plottitle+' - imshow',fontsize=10)
plt.show() # display onscreen.



from __future__ import (absolute_import, division, print_function)

from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
import numpy as np

# 'cubed sphere'
# inscribe the sphere in a cube, then separately project each cube
# face with gnomonic projection.
# http://www.progonos.com/furuti/MapProj/Normal/ProjPoly/Foldout/Cube/cube.html
# suitable for cutting and folding.

# choose figure size to match aspect ratio of map.
fig = plt.figure(figsize=(10,7.5))
fig.subplots_adjust(bottom=0, left=0, right=1, top=1, wspace=0, hspace=0)
rsphere = 6370997.
width = 2.*rsphere; height=width
npanel=0
for lat_0 in [90,0,-90]:
    for ncol in range(0,4):
        npanel = npanel + 1
        if lat_0 == 0 or ncol == 1:
            ax=fig.add_subplot(3,4,npanel)
            ax.set_frame_on(False)
            lon_0=225 + 90*(ncol+1) - 45
            # use fix_aspect=False, so white space won't appear between
            # faces of cube when window is resized.
            m = Basemap(width=width,height=height,resolution=None,\
                        projection='gnom',lon_0=lon_0,lat_0=lat_0,\
                        rsphere=rsphere,fix_aspect=False)
            m.bluemarble(scale=0.5)
            m.drawparallels(np.arange(-90,91,10),color='0.5')
            m.drawmeridians(np.arange(5,365,10),color='0.5')
            #m.drawlsmask(ocean_color='aqua',land_color='coral')
            #m.drawparallels(np.arange(-90,91,10))
            #m.drawmeridians(np.arange(5,365,10))
fig.text(0.625,0.75,\
        'World Map on a Cube\n Gnomonic Projection',\
        fontsize=14)
plt.show()



from mpl_toolkits.basemap import Basemap, cm
# requires netcdf4-python (netcdf4-python.googlecode.com)
from netCDF4 import Dataset as NetCDFFile
import os
import numpy as np
import matplotlib.pyplot as plt

# plot rainfall from NWS using special precipitation
# colormap used by the NWS, and included in basemap.

ncpath = os.path.join(*3 * [".."] + ["examples", "nws_precip_conus_20061222.nc"])
nc = NetCDFFile(ncpath)
# data from http://water.weather.gov/precip/
prcpvar = nc.variables['amountofprecip']
data = 0.01*prcpvar[:]
latcorners = nc.variables['lat'][:]
loncorners = -nc.variables['lon'][:]
lon_0 = -nc.variables['true_lon'].getValue()
lat_0 = nc.variables['true_lat'].getValue()
# create figure and axes instances
fig = plt.figure(figsize=(8,8))
ax = fig.add_axes([0.1,0.1,0.8,0.8])
# create polar stereographic Basemap instance.
m = Basemap(projection='stere',lon_0=lon_0,lat_0=90.,lat_ts=lat_0,\
            llcrnrlat=latcorners[0],urcrnrlat=latcorners[2],\
            llcrnrlon=loncorners[0],urcrnrlon=loncorners[2],\
            rsphere=6371200.,resolution='l',area_thresh=10000)
# draw coastlines, state and country boundaries, edge of map.
m.drawcoastlines()
m.drawstates()
m.drawcountries()
# draw parallels.
parallels = np.arange(0.,90,10.)
m.drawparallels(parallels,labels=[1,0,0,0],fontsize=10)
# draw meridians
meridians = np.arange(180.,360.,10.)
m.drawmeridians(meridians,labels=[0,0,0,1],fontsize=10)
ny = data.shape[0]; nx = data.shape[1]
lons, lats = m.makegrid(nx, ny) # get lat/lons of ny by nx evenly space grid.
x, y = m(lons, lats) # compute map proj coordinates.
# draw filled contours.
clevs = [0,1,2.5,5,7.5,10,15,20,30,40,50,70,100,150,200,250,300,400,500,600,750]
cs = m.contourf(x,y,data,clevs,cmap=cm.s3pcpn)
# add colorbar.
cbar = m.colorbar(cs,location='bottom',pad="5%")
cbar.set_label('mm')
# add title
plt.title(prcpvar.long_name+' for period ending '+prcpvar.dateofdata)
plt.show()



from mpl_toolkits.basemap import Basemap
import numpy as np
import matplotlib.pyplot as plt
# llcrnrlat,llcrnrlon,urcrnrlat,urcrnrlon
# are the lat/lon values of the lower left and upper right corners
# of the map.
# lat_ts is the latitude of true scale.
# resolution = 'c' means use crude resolution coastlines.
m = Basemap(projection='merc',llcrnrlat=-80,urcrnrlat=80,\
            llcrnrlon=-180,urcrnrlon=180,lat_ts=20,resolution='c')
m.drawcoastlines()
m.fillcontinents(color='coral',lake_color='aqua')
# draw parallels and meridians.
m.drawparallels(np.arange(-90.,91.,30.))
m.drawmeridians(np.arange(-180.,181.,60.))
m.drawmapboundary(fill_color='aqua') 
plt.title("Mercator Projection")
plt.show()



#! /usr/bin/env python
"""Helper script to be run by `cibuildwheel` as `before_all` step."""

import os
import sys

HERE = os.path.abspath(__file__)
ROOT = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))
sys.path.insert(0, os.path.join(ROOT))
import utils  # noqa: E402  # pylint: disable=imports


def main():
    """Build the GEOS library based on parsed environment variables."""

    geos_version = os.environ.get("GEOS_VERSION", None)
    if geos_version is None:
        raise ValueError("Undefined environment variable GEOS_VERSION")

    geos_dir = os.environ.get("GEOS_DIR", None)
    if geos_dir is None:
        raise ValueError("Undefined environment variable GEOS_DIR")

    geos_njobs = int(os.environ.get("GEOS_NJOBS", 1))

    # pylint: disable=consider-using-f-string
    print("Running before_all script with the following settings:")
    print("GEOS_DIR: {0}".format(geos_dir))
    print("GEOS_VERSION: {0}".format(geos_version))
    print("GEOS_NJOBS: {0}".format(geos_njobs))

    utils.GeosLibrary(geos_version).build(geos_dir, njobs=geos_njobs)
    return 0


if __name__ == "__main__":
    sys.exit(main())



from __future__ import (absolute_import, division, print_function)

from mpl_toolkits.basemap import Basemap
import numpy as np
import matplotlib.pyplot as plt
import sys

def get_input(prompt):
    if sys.hexversion > 0x03000000:
        return input(prompt)
    else:
        return raw_input(prompt)

# create Basemap instance for Orthographic (satellite view) projection.
lon_0 = float(get_input('enter reference longitude (lon_0):'))
lat_0 = float(get_input('enter reference latitude (lat_0):'))

# map with land/sea mask plotted
fig = plt.figure()
resolution = 'l'; grid = 5
m = Basemap(projection='ortho',lon_0=lon_0,lat_0=lat_0,resolution=resolution)
# land coral, oceans aqua.
# lakes=True means plot inland lakes with ocean color.
# resolution = 5 (default) means use 5 min dataset (can use 2.5)
m.drawmapboundary()
m.drawcoastlines()
m.drawlsmask(land_color='coral',ocean_color='aqua', lakes=True,\
        resolution=resolution,grid=grid)
# draw parallels and meridians.
m.drawparallels(np.arange(-90.,120.,30.))
m.drawmeridians(np.arange(0.,420.,60.))
plt.title('Orthographic Map Centered on Lon=%s, Lat=%s' % (lon_0,lat_0))

# map with continents drawn and filled (continent filling fails for
# lon=-120,lat=60).
fig = plt.figure()
m = Basemap(projection='ortho',lon_0=lon_0,lat_0=lat_0,resolution=resolution)
m.drawcoastlines()
m.fillcontinents(color='coral',lake_color='aqua')
m.drawcountries()
# draw parallels and meridians.
m.drawparallels(np.arange(-90.,120.,30.))
m.drawmeridians(np.arange(0.,420.,60.))
m.drawmapboundary(fill_color='aqua')
# add a map scale.
length = 5000 
x1,y1 = 0.3*m.xmax, 0.25*m.ymax
lon1,lat1 = m(x1,y1,inverse=True)
m.drawmapscale(lon1,lat1,lon1,lat1,length,fontsize=8,barstyle='fancy',\
               labelstyle='fancy',units='km')
plt.title('Orthographic Map Centered on Lon=%s, Lat=%s' % (lon_0,lat_0))
plt.show()



from mpl_toolkits.basemap import Basemap
import numpy as np
import matplotlib.pyplot as plt
# setup polyconic basemap 
# by specifying lat/lon corners and central point.
# area_thresh=1000 means don't plot coastline features less
# than 1000 km^2 in area.
m = Basemap(llcrnrlon=-35.,llcrnrlat=-30,urcrnrlon=80.,urcrnrlat=50.,\
            resolution='l',area_thresh=1000.,projection='poly',\
            lat_0=0.,lon_0=20.)
m.drawcoastlines()
m.fillcontinents(color='coral',lake_color='aqua')
# draw parallels and meridians.
m.drawparallels(np.arange(-80.,81.,20.))
m.drawmeridians(np.arange(-180.,181.,20.))
m.drawmapboundary(fill_color='aqua') 
plt.title("Polyconic Projection")
plt.show()



from __future__ import (absolute_import, division, print_function)

from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
import numpy as np

# test arcgisimage method for retrieving images from web map servers.

plt.figure(figsize=(8,8))
epsg=4326
m=Basemap(projection='cyl',llcrnrlon=-90,llcrnrlat=30,urcrnrlon=-60,urcrnrlat=60,resolution='i')
# default 'blue marble' image.
m.arcgisimage(verbose=True)
m.drawmeridians(np.arange(-180,180,10),labels=[0,0,0,1],color='y')
m.drawparallels(np.arange(-90,90,10),labels=[1,0,0,0],color='y')
m.drawcoastlines(linewidth=0.5,color='0.5')
plt.title('test WMS map background EPSG=%s'%epsg)

plt.figure(figsize=(8,8))
epsg = 3413; width = 18000.e3; height = 18000.e3
m=Basemap(epsg=epsg,resolution='l',width=width,height=height)
# default 'blue marble' image.
m.arcgisimage()
m.drawmeridians(np.arange(-180,180,30),labels=[0,0,0,1],color='y')
m.drawparallels(np.arange(0,80,10),labels=[1,0,0,0],color='y')
m.drawcoastlines(linewidth=0.5,color='0.5')
plt.title('test WMS map background EPSG=%s' % epsg)

plt.figure(figsize=(10,6.6666))
epsg = 2263; width=600.e3; height = 400.e3
m=Basemap(epsg=epsg,resolution='h',width=width,height=height)
# specify a different server.
m.arcgisimage(server='https://gis.ngdc.noaa.gov/arcgis',service='etopo1',verbose=True)
m.drawmeridians(np.arange(-180,180,2),labels=[0,0,0,1])
m.drawparallels(np.arange(0,80,1),labels=[1,0,0,0])
m.drawcoastlines(linewidth=0.25)
plt.title('test WMS map background EPSG=%s' % epsg)

plt.figure(figsize=(6,8))
epsg = 3086; lon1 = -85; lat1 = 24; lon2 = -79; lat2 =32
m=Basemap(epsg=epsg,resolution='i',llcrnrlon=lon1,llcrnrlat=lat1,\
           urcrnrlon=lon2,urcrnrlat=lat2)
# specify a different service
m.arcgisimage(service='NatGeo_World_Map',verbose=True,xpixels=600,interpolation='bicubic')
m.drawmeridians(np.arange(-180,180,2),labels=[0,0,0,1])
m.drawparallels(np.arange(0,80,1),labels=[1,0,0,0])
m.drawcoastlines(linewidth=0.25)
plt.title('test WMS map background EPSG=%s' % epsg)

plt.show()



from __future__ import (absolute_import, division, print_function)

from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
import numpy as np
from numpy import ma
from netCDF4 import Dataset as NetCDFFile
# read in orography of icosahedral global grid.
f = NetCDFFile('C02562.orog.nc')
lons = (180./np.pi)*f.variables['grid_center_lon'][:]
lats = (180./np.pi)*f.variables['grid_center_lat'][:]
z = f.variables['zs'][:]
map = Basemap(projection='ortho',lon_0=-105,lat_0=40)
x,y = map(lons, lats)
map.drawcoastlines()
map.drawmapboundary()
# tri=True forwards to axes.tripcolor
#z = ma.masked_where(z < 1.e-5, z) # for testing masked arrays.
map.pcolor(x,y,z,tri=True,shading='flat',edgecolors='k',cmap=plt.cm.hot_r,vmin=0,vmax=3000)
#map.contourf(x,y,z,np.arange(0,3000,150),tri=True)
#map.contour(x,y,z,np.arange(0,3000,150),tri=True)
plt.title('pcolor plot on a global icosahedral mesh')
plt.show()



import numpy as np
import matplotlib.pyplot as plt
import datetime
from mpl_toolkits.basemap import Basemap, shiftgrid
from netCDF4 import Dataset
# specify date to plot.
yyyy=1993; mm=3; dd=14; hh=0
date = datetime.datetime(yyyy,mm,dd,hh)
# set OpenDAP server URL.
URLbase="https://www.ncei.noaa.gov/thredds/dodsC/model-cfs_reanl_6h_pgb/"
URL=URLbase+"%04i/%04i%02i/%04i%02i%02i/pgbh00.gdas.%04i%02i%02i%02i.grb2" %\
             (yyyy,yyyy,mm,yyyy,mm,dd,yyyy,mm,dd,hh)
data = Dataset(URL)
# read lats,lons
# reverse latitudes so they go from south to north.
latitudes = data.variables['lat'][::-1]
longitudes = data.variables['lon'][:].tolist()
# get sea level pressure and 10-m wind data.
# mult slp by 0.01 to put in units of hPa.
slpin = 0.01*data.variables['Pressure_msl'][:].squeeze()
uin = data.variables['u-component_of_wind_height_above_ground'][:].squeeze()
vin = data.variables['v-component_of_wind_height_above_ground'][:].squeeze()
# add cyclic points manually (could use addcyclic function)
slp = np.zeros((slpin.shape[0],slpin.shape[1]+1),np.float64)
slp[:,0:-1] = slpin[::-1]; slp[:,-1] = slpin[::-1,0]
u = np.zeros((uin.shape[0],uin.shape[1]+1),np.float64)
u[:,0:-1] = uin[::-1]; u[:,-1] = uin[::-1,0]
v = np.zeros((vin.shape[0],vin.shape[1]+1),np.float64)
v[:,0:-1] = vin[::-1]; v[:,-1] = vin[::-1,0]
longitudes.append(360.); longitudes = np.array(longitudes)
# make 2-d grid of lons, lats
lons, lats = np.meshgrid(longitudes,latitudes)
# make orthographic basemap.
m = Basemap(resolution='c',projection='ortho',lat_0=60.,lon_0=-60.)
# create figure, add axes
fig1 = plt.figure(figsize=(8,10))
ax = fig1.add_axes([0.1,0.1,0.8,0.8])
# set desired contour levels.
clevs = np.arange(960,1061,5)
# compute native x,y coordinates of grid.
x, y = m(lons, lats)
# define parallels and meridians to draw.
parallels = np.arange(-80.,90,20.)
meridians = np.arange(0.,360.,20.)
# plot SLP contours.
CS1 = m.contour(x,y,slp,clevs,linewidths=0.5,colors='k')
CS2 = m.contourf(x,y,slp,clevs,cmap=plt.cm.RdBu_r)
# plot wind vectors on projection grid.
# first, shift grid so it goes from -180 to 180 (instead of 0 to 360
# in longitude).  Otherwise, interpolation is messed up.
ugrid,newlons = shiftgrid(180.,u,longitudes,start=False)
vgrid,newlons = shiftgrid(180.,v,longitudes,start=False)
# transform vectors to projection grid.
uproj,vproj,xx,yy = \
m.transform_vector(ugrid,vgrid,newlons,latitudes,31,31,returnxy=True,masked=True)
# now plot.
Q = m.quiver(xx,yy,uproj,vproj,scale=700)
# make quiver key.
qk = plt.quiverkey(Q, 0.1, 0.1, 20, '20 m/s', labelpos='W')
# draw coastlines, parallels, meridians.
m.drawcoastlines(linewidth=1.5)
m.drawparallels(parallels)
m.drawmeridians(meridians)
# add colorbar
cb = m.colorbar(CS2,"bottom", size="5%", pad="2%")
cb.set_label('hPa')
# set plot title
ax.set_title('SLP and Wind Vectors '+str(date))
plt.show()

# create 2nd figure, add axes
fig2 = plt.figure(figsize=(8,10))
ax = fig2.add_axes([0.1,0.1,0.8,0.8])
# plot SLP contours
CS1 = m.contour(x,y,slp,clevs,linewidths=0.5,colors='k')
CS2 = m.contourf(x,y,slp,clevs,cmap=plt.cm.RdBu_r)
# plot wind barbs over map.
barbs = m.barbs(xx,yy,uproj,vproj,length=5,barbcolor='k',flagcolor='r',linewidth=0.5)
# draw coastlines, parallels, meridians.
m.drawcoastlines(linewidth=1.5)
m.drawparallels(parallels)
m.drawmeridians(meridians)
# add colorbar
cb = m.colorbar(CS2,"bottom", size="5%", pad="2%")
cb.set_label('hPa')
# set plot title.
ax.set_title('SLP and Wind Barbs '+str(date))
plt.show()



from mpl_toolkits.basemap import Basemap
from netCDF4 import Dataset, date2index
import numpy as np
import matplotlib.pyplot as plt
from datetime import datetime
try:
    from urllib.request import urlretrieve
except ImportError:
    from urllib import urlretrieve
date = datetime(2007,12,15,0) # date to plot.
# open dataset.
sstpath, sstheader = urlretrieve("https://downloads.psl.noaa.gov/Datasets/noaa.oisst.v2.highres/sst.day.mean.{0}.nc".format(date.year))
dataset = Dataset(sstpath)
timevar = dataset.variables['time']
timeindex = date2index(date,timevar) # find time index for desired date.
# read sst.  Will automatically create a masked array using
# missing_value variable attribute. 'squeeze out' singleton dimensions.
sst = dataset.variables['sst'][timeindex,:].squeeze()
# read ice.
dataset.close()
icepath, iceheader = urlretrieve("https://downloads.psl.noaa.gov/Datasets/noaa.oisst.v2.highres/icec.day.mean.{0}.nc".format(date.year))
dataset = Dataset(icepath)
ice = dataset.variables['icec'][timeindex,:].squeeze()
# read lats and lons (representing centers of grid boxes).
lats = dataset.variables['lat'][:]
lons = dataset.variables['lon'][:]
dataset.close()
latstep, lonstep = np.diff(lats[:2]), np.diff(lons[:2])
lats = np.append(lats - 0.5 * latstep, lats[-1] + 0.5 * latstep)
lons = np.append(lons - 0.5 * lonstep, lons[-1] + 0.5 * lonstep)
lons, lats = np.meshgrid(lons,lats)
# create figure, axes instances.
fig = plt.figure()
ax = fig.add_axes([0.05,0.05,0.9,0.9])
# create Basemap instance.
# coastlines not used, so resolution set to None to skip
# continent processing (this speeds things up a bit)
m = Basemap(projection='kav7',lon_0=0,resolution=None)
# draw line around map projection limb.
# color background of map projection region.
# missing values over land will show up this color.
m.drawmapboundary(fill_color='0.3')
# plot sst, then ice with pcolor
im1 = m.pcolormesh(lons,lats,sst,shading='flat',cmap=plt.cm.jet,latlon=True)
im2 = m.pcolormesh(lons,lats,ice,shading='flat',cmap=plt.cm.gist_gray,latlon=True)
# draw parallels and meridians, but don't bother labelling them.
m.drawparallels(np.arange(-90.,99.,30.))
m.drawmeridians(np.arange(-180.,180.,60.))
# add colorbar
cb = m.colorbar(im1,"bottom", size="5%", pad="2%")
# add a title.
ax.set_title('SST and ICE analysis for %s'%date)
plt.show()



from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
import numpy as np
# setup Lambert Conformal basemap.
m = Basemap(width=12000000,height=9000000,projection='lcc',
            resolution='c',lat_1=45.,lat_2=55,lat_0=50,lon_0=-107.)
# draw coastlines.
m.drawcoastlines()
# draw a boundary around the map, fill the background.
# this background will end up being the ocean color, since
# the continents will be drawn on top.
m.drawmapboundary(fill_color='aqua') 
# fill continents, set lake color same as ocean color. 
m.fillcontinents(color='coral',lake_color='aqua')
# draw parallels and meridians.
# label parallels on right and top
# meridians on bottom and left
parallels = np.arange(0.,81,10.)
# labels = [left,right,top,bottom]
m.drawparallels(parallels,labels=[False,True,True,False])
meridians = np.arange(10.,351.,20.)
m.drawmeridians(meridians,labels=[True,False,False,True])
plt.show()



from __future__ import (absolute_import, division, print_function)

import pyproj
from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
# proj4 definition of UTM zone 17.
# coordinates agree to the 3rd decimal place (less than 1mm)
# when compared to result from
# http://home.hiwaay.net/~taylorc/toolbox/geography/geoutm.html
p1 = pyproj.Proj(proj='utm',zone='17')
miamilon=-80.28; miamilat=25.92
x,y = p1(miamilon,miamilat)
print('utm coordinates from pyproj: ',x,y)
lon_0=-81; lat_0=0.
# mimic this with Basemap (WGS84 ellipsoid, with scaling factor of 0.9996 at
# central meridian).  latitude bounds are given for the MGRS band N, which covers Florida.
m = Basemap(projection='tmerc',lon_0=lon_0,lat_0=lat_0,
            k_0=0.9996,rsphere=(6378137.00,6356752.314245179),
            llcrnrlon=lon_0-3,llcrnrlat=24,urcrnrlon=lon_0+3.,urcrnrlat=32,resolution='i')
x0,y0 = m(lon_0,lat_0) 
x1,y1 = m(miamilon,miamilat)
# add coordinate offsets (Basemap coordinate system has x=0,y=0 at lower
# left corner of domain)
x = x1-x0+5.e5
y = y1-y0
print('utm coordinates from Basemap:',x,y)
m.drawcoastlines()
m.drawstates()
m.fillcontinents(color='coral',lake_color='aqua')
m.drawmapboundary(fill_color='aqua')
m.drawparallels(range(24,33),labels=[1,0,0,0])
m.drawmeridians(range(-84,-77),labels=[0,0,0,1])
plt.title('UTM zone 17N')
plt.show()



from mpl_toolkits.basemap import Basemap
import matplotlib.pyplot as plt
# setup Lambert Conformal basemap.
m = Basemap(width=12000000,height=9000000,projection='lcc',
            resolution='c',lat_1=45.,lat_2=55,lat_0=50,lon_0=-107.)
# draw coastlines.
m.drawcoastlines()
# draw a boundary around the map, fill the background.
# this background will end up being the ocean color, since
# the continents will be drawn on top.
m.drawmapboundary(fill_color='aqua') 
# fill continents, set lake color same as ocean color. 
m.fillcontinents(color='coral',lake_color='aqua')
plt.show()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021-2025 Víctor Molina García
#
# GeosLibrary.py is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# GeosLibrary.py is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with GeosLibrary.py. If not, see <https://www.gnu.org/licenses/>.
#
"""Helper class to download and build the GEOS library."""

import io
import os
import sys
import ssl
import glob
import shutil
import struct
import tempfile
import contextlib
import subprocess
import datetime as dt
from zipfile import ZipFile
try:
    from urllib.request import urlopen
except ImportError:
    from urllib2 import urlopen


URL_DATETIME_FMT = "%a, %d %b %Y %H:%M:%S GMT"
GEOS_BASEURL = "https://github.com/libgeos/geos/archive/refs/tags"


class GeosLibrary(object):
    """Helper class to download, build and install GEOS."""

    def __init__(self, version, root=None):
        """Initialise a new :class:`GeosLibrary` instance."""

        self.version_tuple = tuple(map(int, version.split(".")))

        if root is None:
            self.temp = True
            self.root = tempfile.mkdtemp(prefix="tmp_geoslibrary_")
        else:
            self.temp = False
            self.root = root
            try:
                os.makedirs(self.root)
            except OSError:
                pass

    def __del__(self):
        """Clean up after :class:`GeosLibrary` destruction."""

        if getattr(self, "temp", None) and getattr(self, "root", None):
            try:
                shutil.rmtree(self.root)
            except OSError:
                pass

    @property
    def version(self):
        """GEOS library version in string format."""

        return ".".join(map(str, self.version_tuple))

    def download(self):
        """Download GEOS zip source code into :class:`GeosLibrary` root."""

        # Define download link.
        link = "{0}/{1}.zip".format(GEOS_BASEURL, self.version)
        suffix = os.path.splitext(link)[-1]

        # Define output path.
        zipname = "geos-{0}".format(link.rsplit("/", 1)[-1])
        zippath = os.path.join(self.root, zipname)

        # Handle creation of the HTTP request.
        kwargs = {}
        if hasattr(ssl, "SSLContext") and hasattr(ssl, "PROTOCOL_TLSv1_2"):
            kwargs.update(context=ssl.SSLContext(ssl.PROTOCOL_TLSv1_2))
        try:
            conn = urlopen(link, **kwargs)
        except TypeError:
            # Fallback if `urlopen` does not accept context.
            conn = urlopen(link)

        with contextlib.closing(conn):
            # Try to get the file timestamp from the HTTP request header.
            date = conn.headers.get("Last-Modified")
            if date is not None:
                date = dt.datetime.strptime(date, URL_DATETIME_FMT)
                date = date.replace(tzinfo=dt.timezone.utc)
                date = date.timestamp()
            with tempfile.NamedTemporaryFile(suffix=suffix) as tmpzipobj:
                # Copy the buffer into a temporary file.
                shutil.copyfileobj(conn, tmpzipobj)
                # Move the file descriptor pointer to the beginning and
                # simply copy it to the final destination.
                tmpzipobj.seek(0)
                with open(zippath, "wb") as zipobj:
                    shutil.copyfileobj(tmpzipobj, zipobj)
            # Assign the timestamps to the final file if available.
            if date is not None:
                os.utime(zippath, (date, date))

    def extract(self, overwrite=True):
        """Decompress GEOS zip source code into :class:`GeosLibrary` root."""

        # Download zip file if not present.
        zippath = os.path.join(self.root, "geos-{0}.zip".format(self.version))
        if not os.path.exists(zippath):
            self.download()

        # Remove destination folder if present and requested.
        zipfold = os.path.join(self.root, "geos-{0}".format(self.version))
        if os.path.exists(zipfold):
            if not overwrite:
                raise OSError("folder '{0}' already exists".format(zipfold))
            shutil.rmtree(zipfold)

        # Decompress zip file.
        with contextlib.closing(ZipFile(zippath, "r")) as fd:
            fd.extractall(self.root)

        # Ensure that GEOS internal sh scripts can be executed.
        for path in sorted(glob.glob(os.path.join(zipfold, "tools", "*.sh"))):
            os.chmod(path, 0o755)

        # Apply specific patches for GEOS < 3.6.0.
        if self.version_tuple < (3, 6, 0):
            # The SVN revision file is not created on the fly before 3.6.0.
            svn_hfile = os.path.join(zipfold, "geos_svn_revision.h")
            if not os.path.exists(svn_hfile):
                with io.open(svn_hfile, "wb") as fd:
                    text = "#define GEOS_SVN_REVISION 0"
                    fd.write(text.encode())
            # Reduce warnings when compiling with `nmake` on Windows.
            cmakefile = os.path.join(zipfold, "CMakeLists.txt")
            if os.path.exists(cmakefile):
                with io.open(cmakefile, "r", encoding="utf-8") as fd:
                    lines = fd.readlines()
                with io.open(cmakefile, "wb") as fd:
                    oldtext = 'string(REGEX REPLACE "/W[0-9]" "/W4"'
                    newtext = oldtext.replace("W4", "W1")
                    for line in lines:
                        fd.write(line.replace(oldtext, newtext).encode())

        # Apply specific patches for 3.6.0 <= GEOS < 3.7.0 on Windows.
        if (3, 6, 0) <= self.version_tuple < (3, 7, 0) and os.name == "nt":
            autogen_file = os.path.join(zipfold, "autogen.bat")
            subprocess.call([autogen_file], cwd=zipfold)
            cppfile = os.path.join(zipfold, "src", "geomgraph", "DirectedEdgeStar.cpp")
            with io.open(cppfile, "r", encoding="utf-8") as fd:
                lines = fd.readlines()
            with io.open(cppfile, "wb") as fd:
                oldtext = "DirectedEdgeStar::print() const"
                newtext = oldtext.replace(" const", "")
                for line in lines:
                    fd.write(line.replace(oldtext, newtext).encode())
            hfile = os.path.join(zipfold, "include", "geos", "geomgraph", "DirectedEdgeStar.h")
            with io.open(hfile, "r", encoding="utf-8") as fd:
                lines = fd.readlines()
            with io.open(hfile, "wb") as fd:
                oldtext = "virtual std::string print() const;"
                newtext = oldtext.replace(" const", "")
                for line in lines:
                    fd.write(line.replace(oldtext, newtext).encode())

        # Patch CMakeLists to link shared geos_c with static geos.
        if self.version_tuple < (3, 8, 0):
            cmakefile = os.path.join(zipfold, "capi", "CMakeLists.txt")
            oldtext = "target_link_libraries(geos_c geos)"
            newtext = "target_link_libraries(geos_c geos-static)"
        else:
            cmakefile = os.path.join(zipfold, "CMakeLists.txt")
            oldtext = 'add_library(geos "")'
            newtext = 'add_library(geos STATIC "")'
        with io.open(cmakefile, "r", encoding="utf-8") as fd:
            lines = fd.readlines()
        with io.open(cmakefile, "wb") as fd:
            found_sharedline = False
            shared_oldtext = "if(BUILD_SHARED_LIBS)"
            shared_newtext = "if(FALSE)"
            for line in lines:
                if not found_sharedline and shared_oldtext in line:
                    line = line.replace(shared_oldtext, shared_newtext)
                    found_sharedline = True
                fd.write(line.replace(oldtext, newtext).encode())

        # Patch doc CMakeLists in GEOS 3.8.x series.
        if (3, 8, 0) <= self.version_tuple < (3, 9, 0):
            cmakefile = os.path.join(zipfold, "doc", "CMakeLists.txt")
            oldtext1 = "target_include_directories(test_geos_unit\n"
            newtext1 = "if(BUILD_TESTING)\n    {0}".format(oldtext1)
            oldtext2 = "$<BUILD_INTERFACE:${CMAKE_CURRENT_LIST_DIR}>)\n"
            newtext2 = "{0}endif()\n".format(oldtext2)
            with io.open(cmakefile, "r", encoding="utf-8") as fd:
                lines = fd.readlines()
            with io.open(cmakefile, "wb") as fd:
                for line in lines:
                    line = line.replace(oldtext1, newtext1)
                    line = line.replace(oldtext2, newtext2)
                    fd.write(line.encode())

    def build(self, installdir=None, toolset=None, njobs=1):
        """Build and install GEOS from source."""

        # Download and extract zip file if not present.
        zipfold = os.path.join(self.root, "geos-{0}".format(self.version))
        self.extract(overwrite=True)
        version = self.version_tuple

        # Define build and install directory.
        builddir = os.path.join(zipfold, "build")
        if installdir is None:
            installdir = os.path.expanduser("~/.local/share/libgeos")
        installdir = os.path.abspath(installdir)

        # Define generic configure and build options.
        config_opts = [
            "-DCMAKE_BUILD_TYPE=Release",
            "-DCMAKE_INSTALL_PREFIX={0}".format(installdir),
            "-D{0}=OFF".format("GEOS_ENABLE_TESTS" if version < (3, 8, 0)
                               else "BUILD_TESTING")
        ]
        build_opts = [
            "--config", "Release",
            "--target", "install",
        ]
        build_env = os.environ.copy()

        # Define custom configure and build options.
        if os.name == "nt":
            win64 = (8 * struct.calcsize("P") == 64)
            config_opts += ["-DCMAKE_CXX_FLAGS='/wd4251 /wd4355 /wd4458 /wd4530 /EHsc'"]
            if version >= (3, 6, 0) and sys.version_info[:2] >= (3, 3):
                config_opts = ["-A", "x64" if win64 else "Win32"] + config_opts
                if toolset is not None:
                    try:
                        msvc = "v{0:d}".format(int(float(toolset) * 10))
                    except (TypeError, ValueError):
                        msvc = toolset
                    config_opts += ["-DCMAKE_GENERATOR_TOOLSET={0}".format(msvc)]
                build_opts = ["-j", "{0:d}".format(njobs)] + build_opts
            else:
                config_opts = ["-G", "NMake Makefiles"] + config_opts
                config_opts += ["-DCMAKE_EXE_LINKER_FLAGS='/MANIFEST:NO'"]
                config_opts += ["-DCMAKE_SHARED_LINKER_FLAGS='/MANIFEST:NO'"]
                build_opts.extend([
                    "--",
                    "WIN64={0}".format("YES" if win64 else "NO"),
                    "BUILD_BATCH={0}".format("YES" if njobs > 1 else "NO"),
                ])
                if sys.version_info[:2] < (3, 3):
                    build_opts += ["MSVC_VER=1500"]
        else:
            build_env["MAKEFLAGS"] = "-j {0:d}".format(njobs)
            if version >= (3, 7, 0):
                config_opts += ["-DCMAKE_CXX_FLAGS='-fPIC'"]

        # Call cmake configure after ensuring that the build directory exists.
        try:
            os.makedirs(builddir)
        except OSError:
            pass
        subprocess.call(["cmake", ".."] + config_opts, cwd=builddir)

        # Call cmake build after ensuring that the install directory exists.
        try:
            os.makedirs(installdir)
        except OSError:
            pass
        subprocess.call(["cmake", "--build", "."] + build_opts,
                        cwd=builddir, env=build_env)



from __future__ import (absolute_import, division, print_function)

# make plot of etopo bathymetry/topography data on
# lambert conformal conic map projection, drawing coastlines, state and
# country boundaries, and parallels/meridians.

# the data is interpolated to the native projection grid.

##################################
# pyplot/pylab-free version of plotmap.py
##################################
# set backend to Agg.
import matplotlib, sys
matplotlib.use('Agg')

from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
from mpl_toolkits.basemap import Basemap, shiftgrid
from matplotlib.figure import Figure
import numpy as np
import matplotlib.cm as cm

# read in topo data (on a regular lat/lon grid)
# longitudes go from 20 to 380.
topoin = np.loadtxt('etopo20data.gz')
lons = np.loadtxt('etopo20lons.gz')
lats = np.loadtxt('etopo20lats.gz')
# shift data so lons go from -180 to 180 instead of 20 to 380.
topoin,lons = shiftgrid(180.,topoin,lons,start=False)

# create figure, axes instance.
fig = Figure()
canvas = FigureCanvas(fig)
ax = fig.add_axes([0.1,0.1,0.7,0.7])

# setup of basemap ('lcc' = lambert conformal conic).
# use major and minor sphere radii from WGS84 ellipsoid.
# pass axes instance to Basemap constructor so pylab won't
# be imported.
m = Basemap(llcrnrlon=-145.5,llcrnrlat=1.,urcrnrlon=-2.566,urcrnrlat=46.352,\
            rsphere=(6378137.00,6356752.3142),\
            resolution='l',area_thresh=1000.,projection='lcc',\
            lat_1=50.,lon_0=-107.,ax=ax)
# transform to nx x ny regularly spaced native projection grid
nx = int((m.xmax-m.xmin)/40000.)+1; ny = int((m.ymax-m.ymin)/40000.)+1
topodat,x,y = m.transform_scalar(topoin,lons,lats,nx,ny,returnxy=True)
# plot image over map with imshow.
im = m.imshow(topodat,cm.jet)
# setup colorbar axes instance.
pos = ax.get_position()
l, b, w, h = pos.bounds
cax = fig.add_axes([l+w+0.075, b, 0.05, h],frameon=False) # setup colorbar axes
fig.colorbar(im, cax=cax) # draw colorbar
# plot blue dot on boulder, colorado and label it as such.
xpt,ypt = m(-104.237,40.125) 
m.plot([xpt],[ypt],'bo') 
ax.text(xpt+100000,ypt+100000,'Boulder')
# draw coastlines and political boundaries.
m.drawcoastlines()
m.drawcountries()
m.drawstates()
# draw parallels and meridians.
# label on left, right and bottom of map.
parallels = np.arange(0.,80,20.)
m.drawparallels(parallels,labels=[1,1,0,1])
meridians = np.arange(10.,360.,30.)
m.drawmeridians(meridians,labels=[1,1,0,1])
# set title.
ax.set_title('ETOPO Topography - Lambert Conformal Conic')
# save image (width 800 pixels with dpi=100 and fig width 8 inches).
canvas.print_figure('plotmap',dpi=100)
# done.
sys.stdout.write('image saved in plotmap.png\n')



"""Configuration file for :mod:`mpl_toolkits.basemap` test suite.

The :mod:`mpl_toolkits.basemap` test suite is configured so that all
tests involving :mod:`mpl_toolkits.basemap` are run using the 'Agg'
backend, because there is no guarantee that another backend will be
available at runtime.
"""
from __future__ import absolute_import

import os
import sys

if "MPLBACKEND" not in os.environ:
    os.environ["MPLBACKEND"] = "Agg"

try:
    from mpl_toolkits import basemap
except ImportError:
    import basemap
    sys.modules["mpl_toolkits.basemap"] = basemap



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# flake8: noqa: E122
"""basemap -- Plot data on map projections with matplotlib."""

import io
import os
import re
import sys
import glob
import warnings
from setuptools import setup
from setuptools import find_namespace_packages
from setuptools.command.sdist import sdist
from setuptools.extension import Extension


def get_content(name, splitlines=False):
    """Return the file contents with project root as root folder."""

    here = os.path.abspath(os.path.dirname(__file__))
    path = os.path.join(here, name)
    with io.open(path, "r", encoding="utf-8") as fd:
        content = fd.read()
    if splitlines:
        content = [row for row in content.splitlines() if row]
    return content


def get_version(pkgname):
    """Return package version without importing the file."""

    here = os.path.abspath(os.path.dirname(__file__))
    path = os.path.join(*[here, "src"] + pkgname.split(".") + ["__init__.py"])
    with io.open(path, "r", encoding="utf-8") as fd:
        pattern = r"""\n__version__[ ]*=[ ]*["']([^"]+)["']"""
        return re.search(pattern, fd.read()).group(1)


def get_geos_install_prefix():
    """Return GEOS installation prefix or None if not found."""

    env_candidate = os.environ.get("GEOS_DIR", None)
    if env_candidate is not None:
        candidates = [env_candidate]
    else:
        candidates = [os.path.expanduser("~/local"), os.path.expanduser("~"),
                      "/usr/local", "/usr", "/opt/local", "/opt", "/sw"]

    # Prepare filename pattern to find the GEOS library.
    extensions = {"win32": "dll", "cygwin": "dll", "darwin": "dylib"}
    libext = extensions.get(sys.platform, "so*")
    libname = "*geos_c*.{0}".format(libext)
    libdirs = ["bin", "lib", "lib/x86_64-linux-gnu", "lib64"]

    for prefix in candidates:
        libfiles = []
        for libdir in libdirs:
            libfiles.extend(glob.glob(os.path.join(prefix, libdir, libname)))
        hfile = os.path.join(prefix, "include", "geos_c.h")
        if os.path.isfile(hfile) and libfiles:
            return prefix

    # At this point, the GEOS library was not found, so we throw a warning if
    # the user is trying to build the library.
    build_cmds = ("bdist_wheel", "build", "install")
    if any(cmd in sys.argv[1:] for cmd in build_cmds):
        warnings.warn(" ".join([
            "Cannot find GEOS library and/or headers in standard locations",
            "('{0}'). Please install the corresponding packages using your",
            "software management system or set the environment variable",
            "GEOS_DIR to point to the location where GEOS is installed",
            "(for example, if 'geos_c.h' is in '/usr/local/include'",
            "and 'libgeos_c' is in '/usr/local/lib', then you need to",
            "set GEOS_DIR to '/usr/local'",
        ]).format("', '".join(candidates)), RuntimeWarning)
    return None


class basemap_sdist(sdist):  # pylint: disable=invalid-name
    """Custom `sdist` so that it will not pack DLLs on Windows if present."""

    def run(self):
        """Custom `run` command."""

        # Replace DLL data files and add GEOS build script.
        orig_data_files = self.distribution.data_files
        self.distribution.data_files = [
            (".", glob.glob(os.path.join("utils", "*.py")))]

        # Run the original `run` method and leave `data_files` as it was found.
        try:
            sdist.run(self)
        finally:
            self.distribution.data_files = orig_data_files


# Initialise include and library dirs.
data_files = []
include_dirs = []
library_dirs = []
runtime_library_dirs = []

# Define NumPy include dirs.
numpy_include_path = os.environ.get("NUMPY_INCLUDE_PATH", None)
if numpy_include_path is not None:
    include_dirs.append(numpy_include_path)
else:
    try:
        import numpy
        include_dirs.append(numpy.get_include())
    except ImportError as err:
        cmds = ("bdist_wheel", "build", "install")
        if any(cmd in sys.argv[1:] for cmd in cmds):
            warnings.warn("unable to locate NumPy headers", RuntimeWarning)

# Define GEOS include, library and runtime dirs.
geos_install_prefix = get_geos_install_prefix()
if geos_install_prefix is not None:
    include_dirs.append(os.path.join(geos_install_prefix, "include"))
    library_dirs.append(os.path.join(geos_install_prefix, "lib"))
    library_dirs.append(os.path.join(geos_install_prefix, "lib64"))
    runtime_library_dirs = library_dirs
    if os.name == "nt" or sys.platform == "cygwin":
        # On Windows:
        # - DLLs get installed under `bin`.
        # - We need to inject later the DLL in the wheel using `data_files`.
        # - We do not use `runtime_library_dirs` as workaround for a
        #   `distutils` bug (http://bugs.python.org/issue2437).
        library_dirs.append(os.path.join(geos_install_prefix, "bin"))
        runtime_library_dirs = []
        dlls = glob.glob(os.path.join(geos_install_prefix, "*", "*geos_c*.dll"))
        if dlls:
            data_files.append(("../..", sorted(dlls)))

# Define `_geoslib` extension module. It cannot be installed in the
# `mpl_toolkits.basemap` namespace or `Basemap` objects will not be pickleable.
ext_modules = [
    Extension(**{
        "name":
            "_geoslib",
        "sources": [
            "src/_geoslib.pyx",
        ],
        "libraries": [
            "geos_c",
        ],
        "include_dirs":
            include_dirs,
        "library_dirs":
            library_dirs,
        "runtime_library_dirs":
            runtime_library_dirs,
    }),
]
for ext in ext_modules:
    ext.cython_directives = [
        ("language_level", str(sys.version_info[0])),
    ]

setup(**{
    "name":
        "basemap",
    "version":
        get_version("mpl_toolkits.basemap"),
    "description":
        "Plot data on map projections with matplotlib",
    "long_description":
        get_content("README.md"),
    "long_description_content_type":
        "text/markdown",
    "author":
        "Jeff Whitaker",
    "author_email":
        "jeffrey.s.whitaker@noaa.gov",
    "maintainer":
        "The Matplotlib development team",
    "maintainer_email":
        "matplotlib-users@python.org",
    "license":
        "MIT",
    "license_files": [
        "LICENSE",
        "LICENSE.geos",
    ],
    "classifiers": [
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Visualization",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    "keywords": [
        "GIS",
        "maps",
        "plots",
    ],
    "package_dir":
        {"": "src"},
    "packages":
        find_namespace_packages(where="src"),
    "ext_modules":
        ext_modules,
    "data_files":
        data_files,
    "python_requires":
        ", ".join([
            ">=3.9",
            "<3.14",
        ]),
    "install_requires":
        get_content("dep/requirements.txt", splitlines=True),
    "extras_require": {
        "doc":
            get_content("dep/requirements-doc.txt", splitlines=True),
        "lint":
            get_content("dep/requirements-lint.txt", splitlines=True),
        "test":
            get_content("dep/requirements-test.txt", splitlines=True),
        "owslib":
            get_content("dep/requirements-owslib.txt", splitlines=True),
    },
    "cmdclass": {
        "sdist": basemap_sdist,
    },
    "project_urls": {
        "Homepage":
            "https://github.com/matplotlib/basemap",
        "Documentation":
            "https://matplotlib.org/basemap",
        "Repository":
            "https://github.com/matplotlib/basemap.git",
        "Issues":
            "https://github.com/matplotlib/basemap/issues",
    },
})



from mpl_toolkits.basemap import Basemap
import numpy as np
import matplotlib.pyplot as plt
# create new figure, axes instances.
fig=plt.figure()
ax=fig.add_axes([0.1,0.1,0.8,0.8])
# setup mercator map projection.
m = Basemap(llcrnrlon=-100.,llcrnrlat=20.,urcrnrlon=20.,urcrnrlat=60.,\
            rsphere=(6378137.00,6356752.3142),\
            resolution='l',projection='merc',\
            lat_0=40.,lon_0=-20.,lat_ts=20.)
# nylat, nylon are lat/lon of New York
nylat = 40.78; nylon = -73.98
# lonlat, lonlon are lat/lon of London.
lonlat = 51.53; lonlon = 0.08
# draw great circle route between NY and London
m.drawgreatcircle(nylon,nylat,lonlon,lonlat,linewidth=2,color='b')
m.drawcoastlines()
m.fillcontinents()
# draw parallels
m.drawparallels(np.arange(10,90,20),labels=[1,1,0,1])
# draw meridians
m.drawmeridians(np.arange(-180,180,30),labels=[1,1,0,1])
ax.set_title('Great Circle from New York to London')
plt.show()



from mpl_toolkits.basemap import Basemap
import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
mpl_version = tuple(map(int, mpl.__version__.split(".")))
axkwds = {"axisbg" if mpl_version < (2,) else "facecolor": "k"}

fig = plt.figure()
# global ortho map centered on lon_0,lat_0
lat_0=10.; lon_0=57.
# altitude of camera (in km).
h = 3000.
# resolution = None means don't process the boundary datasets.
m1 = Basemap(projection='nsper',satellite_height=h*1000.,\
        lon_0=lon_0,lat_0=lat_0,resolution=None)
# add an axes with a black background
ax = fig.add_axes([0.1,0.1,0.8,0.8], **axkwds)
# plot just upper right quadrant (corners determined from global map).
# keywords llcrnrx,llcrnry,urcrnrx,urcrnry used to define the lower
# left and upper right corners in map projection coordinates.
# llcrnrlat,llcrnrlon,urcrnrlon,urcrnrlat could be used to define
# lat/lon values of corners - but this won't work in cases such as this
# where one of the corners does not lie on the earth.
m = Basemap(projection='nsper',satellite_height=h*1000.,\
        lon_0=lon_0,lat_0=lat_0,resolution='l',\
    llcrnrx=0.,llcrnry=0.,urcrnrx=m1.urcrnrx/2.,urcrnry=m1.urcrnry/2.)
m.drawcoastlines()
m.drawmapboundary(fill_color='aqua')
m.fillcontinents(color='coral',lake_color='aqua')
m.drawcountries()
# draw parallels and meridians.
m.drawparallels(np.arange(-90.,120.,30.))
m.drawmeridians(np.arange(0.,360.,60.))
m.drawmapboundary()
plt.title('Near-Sided Perspective Map Showing A Quadrant of the Globe',\
          fontsize=12)
plt.show()
